# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD13

Description:
This module downloads MOD13 NDVI data from
http://e4ftl01.cr.usgs.gov/. Use the MOD13.NDVI_16daily function to
download and create 16 daily NDVI images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from watertools.Collect import MOD13
MOD13.NDVI_16daily(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .NDVI_16daily import main as NDVI_16daily

__all__ = ['NDVI_16daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/Globcover


Description:
This module downloads Landuse data from http://due.esrin.esa.int/page_globcover.php.
Use the Globcover functions to download and create landuse images in Gtiff format.

Examples:
from watertools.Collect import Globcover
Globcover.Landuse(Dir='C:/TempDEM4/', latlim=[29, 32], lonlim=[-113, -109])
"""
from .Landuse import main as Landuse

__all__ = ['Landuse']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MCD19

Description:
This module downloads MCD19 reflectance data from
https://e4ftl01.cr.usgs.gov/. Use the MCD19.Albedo_8daily function to
download and create 8-daily Reflectance images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from watertools.Collect import MCD19
MCD19.Albedo_8daily(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .Albedo_8daily import main as Albedo_8daily

__all__ = ['Albedo_8daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
    Authors: Tim Hessels
    Module: Products/ETref

    Description:
    This module creates ETref data. The data will first be collected by downloading
    CFSR, HydroSHED, and GLDAS data.
    The LANDSAF data can also be used by using the LANDSAF=1 code. The downloading
    of LANDSAF data is not automated, so you have to download this data by yourself
    and define the path to this data in the SoureLANDSAF parameter.

    LANDSAF:
    The LANDSAF data can be downloaded from CMSAF.eu. You need to have the following directories:
    ~LANDSAF/SIS/SISdm200501310000001190010801GL.nc.gz
    or
    ~LANDSAF/SIS/SISdm20140903000000123MVMFG01ME.nc.gz
    and
    ~LANDSAF/SID/DNIdm200501010000002231000101MA.nc.gz
    or
    ~LANDSAF/SID/SIDdm20140903000000123MVMFG01ME.nc.gz

    The ~LANDSAF is the path which you have to define as the SourceLANDSAF parameter

    ETref.monthly(Dir='C:/TempR/', Startdate='2012-09-01', Enddate='2012-09-02',
               latlim=[33, 35], lonlim=[35, 37], pixel_size=0.01)
    ETref.daily(Dir="C:/TempTestLANDSAF3/", Startdate='2014-09-01', Enddate='2014-09-03',
               latlim=[27, 28], lonlim=[29, 30], pixel_size = 0.01, LANDSAF=1, SourceLANDSAF =r"C:/Users/tih/Documents/Water_Accounting/Landsaf")
"""

from watertools.Products.ETref.daily import main as daily
from watertools.Products.ETref.monthly import main as monthly

__all__ = ['daily', 'monthly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Module: Collect/CFSR
"""
# General modules
import pandas as pd
import os
import numpy as np
from netCDF4 import Dataset
import re
from joblib import Parallel, delayed

# WA+ modules
from watertools.Collect.CFSR.Download_data_CFSR import Download_data
from watertools.General import data_conversions as DC

def CollectData(Dir, Var, Startdate, Enddate, latlim, lonlim, Waitbar, cores, Version):
    """
    This function collects daily CFSR data in geotiff format

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Var -- 'dlwsfc','dswsfc','ulwsfc', or 'uswsfc'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    Waitbar -- 1 (Default) will print a wait bar
    cores -- The number of cores used to run the routine.
             It can be 'False' to avoid using parallel computing
		    routines.
    Version -- 1 or 2 (1 = CFSR, 2 = CFSRv2)
    """


    # Creates an array of the days of which the ET is taken
    Dates = pd.date_range(Startdate,Enddate,freq = 'D')

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)


    # For collecting CFSR data
    if Version == 1:
        # Check the latitude and longitude and otherwise set lat or lon on greatest extent
        if latlim[0] < -89.9171038899 or latlim[1] > 89.9171038899:
            print('Latitude above 89.917N or below 89.917S is not possible. Value set to maximum')
            latlim[0] = np.maximum(latlim[0],-89.9171038899)
            latlim[1] = np.minimum(latlim[1],89.9171038899)
        if lonlim[0] < -180 or lonlim[1] > 179.843249782:
            print('Longitude must be between 179.84E and 179.84W. Now value is set to maximum')
            lonlim[0] = np.maximum(lonlim[0],-180)
            lonlim[1] = np.minimum(lonlim[1],179.843249782)

        # Make directory for the CFSR data
        output_folder=os.path.join(Dir,'Radiation','CFSR')
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)

    # For collecting CFSRv2 data
    if Version == 2:
            # Check the latitude and longitude and otherwise set lat or lon on greatest extent
        if latlim[0] < -89.9462116040955806 or latlim[1] > 89.9462116040955806:
            print('Latitude above 89.917N or below 89.946S is not possible. Value set to maximum')
            latlim[0] = np.maximum(latlim[0],-89.9462116040955806)
            latlim[1] = np.minimum(latlim[1],89.9462116040955806)
        if lonlim[0] < -180 or lonlim[1] > 179.8977275:
            print('Longitude must be between 179.90E and 179.90W. Now value is set to maximum')
            lonlim[0] = np.maximum(lonlim[0],-180)
            lonlim[1] = np.minimum(lonlim[1],179.8977275)

        # Make directory for the CFSRv2 data
        output_folder=os.path.join(Dir,'Radiation','CFSRv2')
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)


    # Pass variables to parallel function and run
    args = [output_folder, latlim, lonlim, Var, Version]
    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)

    # Remove all .nc and .grb2 files
    for f in os.listdir(output_folder):
        if re.search(".nc", f):
            os.remove(os.path.join(output_folder, f))
    for f in os.listdir(output_folder):
        if re.search(".grb2", f):
            os.remove(os.path.join(output_folder, f))
    for f in os.listdir(output_folder):
        if re.search(".grib2", f):
            os.remove(os.path.join(output_folder, f))

    return results

def RetrieveData(Date, args):

    # unpack the arguments
    [output_folder, latlim, lonlim, Var, Version] = args

    # Name of the model
    if Version == 1:
        version_name = 'CFSR'
    if Version == 2:
        version_name = 'CFSRv2'

    # Name of the outputfile
    if Var == 'dlwsfc':
        Outputname = 'DLWR_%s_W-m2_' %version_name + str(Date.strftime('%Y')) + '.' + str(Date.strftime('%m')) + '.' + str(Date.strftime('%d')) + '.tif'
    if Var == 'dswsfc':
        Outputname = 'DSWR_%s_W-m2_' %version_name + str(Date.strftime('%Y')) + '.' + str(Date.strftime('%m')) + '.' + str(Date.strftime('%d')) + '.tif'
    if Var == 'ulwsfc':
        Outputname = 'ULWR_%s_W-m2_' %version_name + str(Date.strftime('%Y')) + '.' + str(Date.strftime('%m')) + '.' + str(Date.strftime('%d')) + '.tif'
    if Var == 'uswsfc':
        Outputname = 'USWR_%s_W-m2_' %version_name + str(Date.strftime('%Y')) + '.' + str(Date.strftime('%m')) + '.' + str(Date.strftime('%d')) + '.tif'

    # Create the total end output name
    outputnamePath = os.path.join(output_folder, Outputname)

    # If the output name not exists than create this output
    if not os.path.exists(outputnamePath):


        local_filename = Download_data(Date, Version, output_folder, Var)

        # convert grb2 to netcdf (wgrib2 module is needed)
        for i in range(0,4):
            nameNC = 'Output' + str(Date.strftime('%Y')) + str(Date.strftime('%m')) + str(Date.strftime('%d')) + '-' + str(i+1) + '.nc'

            # Total path of the output
            FileNC6hour = os.path.join(output_folder, nameNC)

	       # Band number of the grib data which is converted in .nc
            band=(int(Date.strftime('%d')) - 1) * 28 + (i + 1) * 7

            # Convert the data
            DC.Convert_grb2_to_nc(local_filename, FileNC6hour, band)

        if Version == 1:

            if Date < pd.Timestamp(pd.datetime(2011, 1, 1)):

                # Convert the latlim and lonlim into array
                Xstart = np.floor((lonlim[0] + 180.1562497) / 0.3125)
                Xend = np.ceil((lonlim[1] + 180.1562497) / 0.3125) + 1
                Ystart = np.floor((latlim[0] + 89.9171038899) / 0.3122121663)
                Yend = np.ceil((latlim[1] + 89.9171038899) / 0.3122121663)

                # Create a new dataset
                Datatot = np.zeros([576, 1152])

            else:
                Version = 2

        if Version == 2:

            # Convert the latlim and lonlim into array
            Xstart = np.floor((lonlim[0] + 180.102272725) / 0.204545)
            Xend = np.ceil((lonlim[1] + 180.102272725) / 0.204545) + 1
            Ystart = np.floor((latlim[0] + 89.9462116040955806) / 0.204423)
            Yend = np.ceil((latlim[1] + 89.9462116040955806) / 0.204423)

            # Create a new dataset
            Datatot = np.zeros([880, 1760])

        # Open 4 times 6 hourly dataset
        for i in range (0, 4):
            nameNC = 'Output' + str(Date.strftime('%Y')) + str(Date.strftime('%m')) + str(Date.strftime('%d')) + '-' + str(i + 1) + '.nc'
            FileNC6hour = os.path.join(output_folder, nameNC)
            f = Dataset(FileNC6hour, mode = 'r')
            Data = f.variables['Band1'][0:int(Datatot.shape[0]), 0:int(Datatot.shape[1])]
            f.close()
            data = np.array(Data)
            Datatot = Datatot + data

        # Calculate the average in W/m^2 over the day
        DatatotDay = Datatot / 4
        DatatotDayEnd = np.zeros([int(Datatot.shape[0]), int(Datatot.shape[1])])
        DatatotDayEnd[:,0:int(Datatot.shape[0])] = DatatotDay[:, int(Datatot.shape[0]):int(Datatot.shape[1])]
        DatatotDayEnd[:,int(Datatot.shape[0]):int(Datatot.shape[1])] = DatatotDay[:, 0:int(Datatot.shape[0])]

        # clip the data to the extent difined by the user
        DatasetEnd = DatatotDayEnd[int(Ystart):int(Yend), int(Xstart):int(Xend)]

        # save file
        if Version == 1:
	        pixel_size = 0.3125
        if Version == 2:
	        pixel_size = 0.204545
        geo = [lonlim[0],pixel_size,0,latlim[1],0,-pixel_size]
        DC.Save_as_tiff(data = np.flipud(DatasetEnd), name = outputnamePath, geo = geo, projection = "WGS84")

    return()



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 24 09:54:37 2017

@author: tih
"""

 
def API(output_folder, DownloadType, string1, string2, string3, string4, string5, string6, string7, string8, string9, string10):

    import os
    from ecmwfapi import ECMWFDataServer
    os.chdir(output_folder)	
    server = ECMWFDataServer()

    if DownloadType == 1 or DownloadType == 2:
        server.retrieve({
            'stream'    : "%s" %string1,
            'levtype'   : "%s" %string2,
            'param'     : "%s" %string3,
            'dataset'   : "interim",
            'step'      : "%s" %string4,
            'grid'      : "%s" %string5,
            'time'      : "%s" %string6,
            'date'      : "%s" %string7,
            'type'      : "%s" %string8,     # http://apps.ecmwf.int/codes/grib/format/mars/type/
            'class'     : "%s" %string9,     # http://apps.ecmwf.int/codes/grib/format/mars/class/
            'area'      : "%s" %string10,   							
            'format'    : "netcdf",
            'target'    : "data_interim.nc"
            })

    if DownloadType == 3:
        server.retrieve({
            'levelist'   : "1000",
            'stream'    : "%s" %string1,
            'levtype'   : "%s" %string2,
            'param'     : "%s" %string3,
            'dataset'   : "interim",
            'step'      : "%s" %string4,
            'grid'      : "%s" %string5,
            'time'      : "%s" %string6,
            'date'      : "%s" %string7,
            'type'      : "%s" %string8,     # http://apps.ecmwf.int/codes/grib/format/mars/type/
            'class'     : "%s" %string9,     # http://apps.ecmwf.int/codes/grib/format/mars/class/
            'area'      : "%s" %string10,   							
            'format'    : "netcdf",
            'target'    : "data_interim.nc"
            })
	
	
    return()
	
	
	
	
	


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/NLDAS


Description:
This script automatically downloads NLDAS data with a 0.125 degree
resolution for different extends based on their opendap server. The time
interval available are: three-hourly ('3hour'), daily ('day'), and monthly
('month'). A list of the variables can be printed with the command:
    NLDAS.VarInfo('daily').descriptions.keys()
Futher information of the variable can be printed with the following commands:
    NLDAS.VarInfo('daily').descriptions['evap']
    NLDAS.VarInfo('daily').units['evap']
    NLDAS.VarInfo('daily').names['evap']
    NLDAS.VarInfo('daily').factors['evap']

Examples:
from watertools.Collect import NLDAS
NLDAS.FORA_hourly(Dir='C:/Temp/', Vars=['tmp2m','pressfc'], Startdate='2004-12-20', Enddate='2005-01-10',
                   latlim=[38, 41], lonlim=[-76, -73], Periods=[4, 5])
NLDAS.FORA_daily(Dir='C:/Temp/', Vars=['tmp2m'], Startdate='2004-12-20', Enddate='2005-01-01',
            latlim=[38, 41], lonlim=[-76, -73],
            SumMean=1, Min=1, Max=1)
NLDAS.FORA_monthly(Dir='C:/TempNLDAS', Vars=['tmp2m'], Startdate='2004-12-20', Enddate='2005-03-10',latlim=[38, 41], lonlim=[-76, -73])
"""

from .FORA_DataAccess import VariablesInfo as FORA_VarInfo
from .FORA_daily import main as FORA_daily
from .FORA_monthly import main as FORA_monthly
from .FORA_hourly import main as FORA_hourly
from .DataAccess import VariablesInfo as VarInfo
from .daily import main as daily
from .monthly import main as monthly
from .hourly import main as hourly

__all__ = ['FORA_VarInfo', 'FORA_daily', 'FORA_monthly', 'FORA_hourly', 'VarInfo', 'daily', 'monthly', 'hourly']

__version__ = '0.1'



# -*- coding: utf-8 -*-

# General modules
import shutil
import os
import urllib

# Water Accounting modules
import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC

def DownloadData(Dir, latlim, lonlim, Waitbar):
    """
    This function downloads NLDAS Forcing data hourly, daily or monthly data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    """
    
    # Define the output name
    output_filename = os.path.join(Dir, 'LU_ESACCI.tif')
    
    # Set the url of the server
    url = r"https://storage.googleapis.com/cci-lc-v207/ESACCI-LC-L4-LCCS-Map-300m-P1Y-2015-v2.0.7.zip"
    
    # Create a Trash folder
    Dir_trash = os.path.join(Dir, "Trash")
    if not os.path.exists(Dir_trash):
        os.makedirs(Dir_trash)
    
    # Define location of download
    filename_out = os.path.join(Dir_trash, "ESACCI-LC-L4-LCCS-Map-300m-P1Y-2015-v2.0.7.zip")

    # Download data
    urllib.request.urlretrieve(url, filename=filename_out)
    
    # Extract data
    DC.Extract_Data(filename_out, Dir_trash)
    
    # Define input of the world tiff file
    filename_world = os.path.join(Dir_trash, "product", "ESACCI-LC-L4-LCCS-Map-300m-P1Y-2015-v2.0.7.tif")
    
    try:
        # Clip data to user extend
        data, Geo_out = RC.clip_data(filename_world, latlim, lonlim)
        
        # Save data of clipped array
        DC.Save_as_tiff(output_filename, data, Geo_out, 4326)
        
    except:
        
        RC.Clip_Dataset_GDAL(RC.clip_data(filename_world, latlim, lonlim))
    
    # Remove trash folder
    shutil.rmtree(Dir_trash)

    return()


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/DEM


Description:
This module downloads DEM data from http://earlywarning.usgs.gov/hydrodata/.
Use the DEM functions to download and create DEM images in Gtiff format.

Examples:
from watertools.Collect import DEM
DEM.HydroSHED(Dir='C:/TempDEM4/', latlim=[29, 32], lonlim=[-113, -109])
"""
from .HydroSHED import main as HydroSHED
from .HydroSHED_Dir import main as HydroSHED_Dir
from .HydroSHED_Acc import main as HydroSHED_Acc
from .SRTM import main as SRTM

__all__ = ['HydroSHED','HydroSHED_Dir', 'HydroSHED_Acc', 'SRTM']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Function/Start
"""
# import general python modules
import numpy as np
import os

# import WA modules
from watertools.General import raster_conversions as RC

def Degrees_to_m2(Reference_data):
    """
    This functions calculated the area of each pixel in squared meter.

    Parameters
    ----------
    Reference_data: str
        Path to a tiff file or nc file or memory file of which the pixel area must be defined

    Returns
    -------
    area_in_m2: array
        Array containing the area of each pixel in squared meters

    """
    try:
        # Get the extension of the example data
        filename, file_extension = os.path.splitext(Reference_data)

        # Get raster information
        if str(file_extension) == '.tif':
            geo_out, proj, size_X, size_Y = RC.Open_array_info(Reference_data)
        elif str(file_extension) == '.nc':
            geo_out, epsg, size_X, size_Y, size_Z, Time = RC.Open_nc_info(Reference_data)

    except:
        geo_out = Reference_data.GetGeoTransform()
        size_X = Reference_data.RasterXSize
        size_Y = Reference_data.RasterYSize

    # Calculate the difference in latitude and longitude in meters
    dlat, dlon = Calc_dlat_dlon(geo_out, size_X, size_Y)

    # Calculate the area in squared meters
    area_in_m2 =  dlat * dlon

    return(area_in_m2)

def Calc_dlat_dlon(geo_out, size_X, size_Y):
    """
    This functions calculated the distance between each pixel in meter.

    Parameters
    ----------
    geo_out: array
        geo transform function of the array
    size_X: int
        size of the X axis
    size_Y: int
        size of the Y axis

    Returns
    -------
    dlat: array
        Array containing the vertical distance between each pixel in meters
    dlon: array
        Array containing the horizontal distance between each pixel in meters
    """

    # Create the lat/lon rasters
    lon = np.arange(size_X + 1)*geo_out[1]+geo_out[0] - 0.5 * geo_out[1]
    lat = np.arange(size_Y + 1)*geo_out[5]+geo_out[3] - 0.5 * geo_out[5]

    dlat_2d = np.array([lat,]*int(np.size(lon,0))).transpose()
    dlon_2d =  np.array([lon,]*int(np.size(lat,0)))

    # Radius of the earth in meters
    R_earth = 6371000

    # Calculate the lat and lon in radians
    lonRad = dlon_2d * np.pi/180
    latRad = dlat_2d * np.pi/180

    # Calculate the difference in lat and lon
    lonRad_dif = abs(lonRad[:,1:] - lonRad[:,:-1])
    latRad_dif = abs(latRad[:-1] - latRad[1:])

    # Calculate the distance between the upper and lower pixel edge
    a = np.sin(latRad_dif[:,:-1]/2) * np.sin(latRad_dif[:,:-1]/2)
    clat = 2 * np.arctan2(np.sqrt(a), np.sqrt(1-a));
    dlat = R_earth * clat

    # Calculate the distance between the eastern and western pixel edge
    b = np.cos(latRad[1:,:-1]) * np.cos(latRad[:-1,:-1]) * np.sin(lonRad_dif[:-1,:]/2) * np.sin(lonRad_dif[:-1,:]/2)
    clon = 2 * np.arctan2(np.sqrt(b), np.sqrt(1-b));
    dlon = R_earth * clon

    return(dlat, dlon)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/HiHydroSoil

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the HiHydroSoil developers.

Description:
This script collects HiHydroSoil data from the UNESCO-IHE FTP server. The
resulting tiff files are in the WGS84 projection.

Example:
from watertools.Collect import HiHydroSoil
HiHydroSoil.ThetaSat(Dir='C:/Temp/', latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
from ftplib import FTP

# Water Accounting Modules
import watertools
import watertools.General.data_conversions as DC

def DownloadData(Dir, latlim, lonlim, Waitbar):
    """
    This scripts downloads HiHydroSoil Saturated Theta soil data from the UNESCO-IHE ftp server.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    lonlim -- [ymin, ymax] (values must be between -90 and 90)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -90 or latlim[1] > 90:
        print('Latitude above 90N or below -90S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0], -90)
        latlim[1] = np.min(latlim[1], 90)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0],-180)
        lonlim[1] = np.min(lonlim[1],180)

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        amount = 0
        WaitbarConsole.printWaitBar(amount, 1, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Define directory and create it if not exists
    output_folder = os.path.join(Dir, 'HiHydroSoil', 'ThetaSat')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Date as printed in filename
    Filename_out= os.path.join(output_folder,'Theta_Saturated_Topsoil_HiHydroSoil.tif')

    # Define end filename
    Filename_in = os.path.join("wcsat_topsoil.tif")

		 # Temporary filename for the downloaded global file
    local_filename = os.path.join(output_folder, Filename_in)

    if not os.path.exists(Filename_out):

        # Download the data from FTP server if the file not exists
        try:
            if not os.path.exists(local_filename):
                Download_HiHydroSoil_from_WA_FTP(local_filename, Filename_in)
            else:
                statinfo = os.stat(local_filename)
                if int(statinfo.st_size) < 3746200000:    
                    Download_HiHydroSoil_from_WA_FTP(local_filename, Filename_in)                
                    
            # Clip dataset
            Clip_Dataset(local_filename, Filename_out, latlim, lonlim)
            os.remove(local_filename)
    
        except:
            print("Was not able to download file")
        
    # Adjust waitbar
    if Waitbar == 1:
        amount += 1
        WaitbarConsole.printWaitBar(amount, 1, prefix = 'Progress:', suffix = 'Complete', length = 50)

    return

def Download_HiHydroSoil_from_WA_FTP(local_filename, Filename_in):
    """
    This function retrieves HiHydroSoil data for a given date from the
    ftp.wateraccounting.unesco-ihe.org server.

    Restrictions:
    The data and this python file may not be distributed to others without
    permission of the WA+ team due data restriction of the HiHydroSoil developers.

    Keyword arguments:
	 local_filename -- name of the temporary file which contains global HiHydroSoil data
    Filename_in -- name of the end file with the HiHydroSoil data
    """

    # Collect account and FTP information
    username, password = watertools.Functions.Random.Get_Username_PWD.GET("FTP_WA")
    ftpserver = "ftp.wateraccounting.unesco-ihe.org"

    # Download data from FTP
    ftp=FTP(ftpserver)
    ftp.login(username,password)
    directory="/WaterAccounting_Guest/Static_WA_Datasets/"
    ftp.cwd(directory)
    lf = open(local_filename, "wb")
    ftp.retrbinary("RETR " + Filename_in, lf.write)
    lf.close()

    return

def Clip_Dataset(local_filename, Filename_out, latlim, lonlim):

    import watertools.General.raster_conversions as RC

    # Open Dataset
    HiHydroSoil_Array = RC.Open_tiff_array(local_filename)

    # Define area
    XID = [int(np.floor((180 + lonlim[0])/0.00833333)), int(np.ceil((180 + lonlim[1])/0.00833333))]
    YID = [int(np.ceil((90 - latlim[1])/0.00833333)), int(np.floor((90 -  latlim[0])/0.00833333))]

    # Define Georeference
    geo = tuple([-180 + 0.00833333*XID[0],0.00833333,0,90 - 0.00833333*YID[0],0,-0.00833333])

    # Clip Array
    HiHydroSoil_Array_clipped = HiHydroSoil_Array[YID[0]:YID[1], XID[0]:XID[1]]

    # Save tiff file
    DC.Save_as_tiff(Filename_out, HiHydroSoil_Array_clipped, geo, "WGS84")



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect/SoilGrids


Description:


Examples:



"""
from .Bulk_Density import main as Bulk_Density
from .Clay_Content import main as Clay_Content
from .Organic_Carbon_Content import main as Organic_Carbon_Content
from .Organic_Carbon_Density import main as Organic_Carbon_Density
from .Sand_Content import main as Sand_Content
from .Silt_Content import main as Silt_Content
from .Nitrogen import main as Nitrogen
from .Soil_pH import main as Soil_pH

__all__ = ['Bulk_Density','Clay_Content','Organic_Carbon_Content','Organic_Carbon_Density','Sand_Content','Silt_Content','Nitrogen','Soil_pH']

__version__ = '0.1'



import sys
from watertools.Collect.MOD13.DataAccess import DownloadData


def main(Dir, Startdate, Enddate, latlim, lonlim, cores=False, Waitbar = 1, hdf_library = None, remove_hdf = 1):
    """
    This function downloads MOD13 16-daily data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
	 cores -- amount of cores used
    Waitbar -- 1 (Default) will print a waitbar
    """
    print('\nDownload 16-daily MODIS NDVI data for period %s till %s' %(Startdate, Enddate))
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores, hdf_library, remove_hdf)

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/PROBAV

Description:
This script collects PROBAV data from the VITO server. The data has a
daily temporal resolution and a spatial resolution of 0.000996 degree. The
resulting tiff files are in the WGS84 projection.
The data is available between 2014-03-11 till present.

Example:
from watertools.Collect import PROBAV
PROBAV.5_daily(Dir='C:/Temp/', Startdate='2014-03-11', Enddate='2015-03-11',
                     latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
import pandas as pd
import requests
from requests.auth import HTTPBasicAuth
import shutil
from osgeo import gdal
import re
from bs4 import BeautifulSoup

# Water Accounting Modules
import watertools
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, type_PROBAV, type_Bands = ['SM','B1','B2','B3','B4']):
    """
    This scripts downloads PROBAV data from the VITO server.
    The output files display the defined bands of PROBA-V.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    lonlim -- [ymin, ymax]
    latlim -- [xmin, xmax]
    """

    # Check the latitude and longitude and otherwise reset lat and lon.
    if latlim[0] < -65 or latlim[1] > 75:
        print('Latitude above 65N or below 75S is not possible.\
            Value set to maximum')
        latlim[0] = np.max(latlim[0], -65)
        latlim[1] = np.min(latlim[1], 75)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W.\
            Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Check Startdate and Enddate
    if not Startdate:
        Startdate = pd.Timestamp('2014-03-11')
    if not Enddate:
        Enddate = pd.Timestamp.now()

    # Define some parameters for the different set of PROBA-V Parameters
    if type_PROBAV.split('_')[-1] == "S10":
        type_time = "10-daily"
        Startdate_pandas = pd.Timestamp(Startdate)
        Day = Startdate_pandas.day
        Day_offset = int(np.ceil(((Day -1) / 10. - int(int(Day - 1) / 5)) * 10))  
        Startdate = Startdate_pandas - pd.offsets.Day(Day_offset)
        list_days = [1,11,21]
    elif type_PROBAV.split('_')[-1] == "S5":
        type_time = "5-daily"
        Startdate_pandas = pd.Timestamp(Startdate)
        Day = Startdate_pandas.day
        Day_offset = int(np.ceil(((Day -1) / 5. - int(int(Day - 1) / 5)) * 5)) 
        Startdate = Startdate_pandas - pd.offsets.Day(Day_offset)
        list_days = [1,6,11,16,21,26]
    elif type_PROBAV.split('_')[-1] == "S1":
        type_time = "daily"
        
    # amount of Dates weekly
    Dates = pd.date_range(Startdate, Enddate, freq="D")
    if type_time != "daily":
        Dates = [date for date in Dates if date.day in list_days]

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix='Progress:',
                                    suffix='Complete', length=50)

    # Define directory and create it if not exists
    output_folder = os.path.join(Dir, '%s'%(str(type_PROBAV.split('_')[0])), 'PROBAV', '%s' %type_time)
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    output_folder_temp = os.path.join(Dir, '%s'%(str(type_PROBAV.split('_')[0])), 'PROBAV', '%s' %type_time, 'Temp')
    if not os.path.exists(output_folder_temp):
        os.makedirs(output_folder_temp)

    # loop over dates
    for Date in Dates:

        # Loop over all the required bands
        for type_Band in type_Bands:
        
            # Define end filename
            End_filename = os.path.join(output_folder, '%s_PROBAV_-_%s_%d.%02d.%02d_%s.tif' %(type_PROBAV, type_time, Date.year, Date.month, Date.day, type_Band))
    
            # Download the data from FTP server if the file not exists
            if not os.path.exists(End_filename):
                try:
                    # Download the PROBA-V Data
                    output_files_PROBAV = Download_PROBAV_from_VITO(End_filename,
                                                    output_folder_temp, Date, latlim, lonlim, type_PROBAV, type_Band)

                    # Create dataset
                    Data, Geo_out = Create_Array(output_files_PROBAV, latlim, lonlim)
                    
                    # Save Data
                    DC.Save_as_tiff(End_filename, Data, Geo_out, "WGS84")
    
                except:
                    print("Was not able to create PROBAV file with date %s" % Date)

        # Adjust waitbar
        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount,
                                        prefix='Progress:', suffix='Complete',
                                        length=50)

    # remove the temporary folder
    #shutil.rmtree(output_folder_temp)
    
    return()


def Download_PROBAV_from_VITO(End_filename, output_folder_temp, Date, latlim, lonlim, type_PROBAV, type_Band):
    """
    This function retrieves PROBAV data for a given date from the
    https://www.vito-eodata.be/PDF/datapool/Free_Data server.

    Restrictions:
    Sign up is needed. The password and username has to be stored within the
    WebAccounts.py

    Keyword arguments
    """

    # Define date
    year_data = Date.year
    month_data = Date.month
    day_data = Date.day

    # Define Tiles
    X_tile_max = int(np.floor((lonlim[1] + 180) / 10.))
    X_tile_min = int(np.floor((lonlim[0] + 180) / 10.))
    Y_tile_min = int(14 - np.ceil((latlim[1] + 65) / 10.))
    Y_tile_max = int(14 - np.ceil((latlim[0] + 65) / 10.))
    X_tiles = list(range(X_tile_min, X_tile_max+1))
    Y_tiles = list(range(Y_tile_min, Y_tile_max+1))
    output_files_PROBAV = []
    
    # Band numbers PROBA-V
    Band_numbers = {'SM':7,'B1':8,'B2':10,'B3':9,'B4':11}
 
    # Get passwords
    username, password = watertools.Functions.Random.Get_Username_PWD.GET('VITO')
    
    # Download all the needed tiles
    for X_tile in X_tiles:
        for Y_tile in Y_tiles:
    
            # filename of ASCAT data on server
            PROBAV_date = "%d%02d%02d" % (year_data, month_data, day_data)
            URL_name = "https://www.vito-eodata.be/PDF/datapool/Free_Data/PROBA-V_100m/%s_%s_100_m_C1/%s/%s/%s/" %(type_PROBAV.split('_')[-1], type_PROBAV.split('_')[0], year_data, month_data, day_data)
        
            # Get version name
            f = requests.get(URL_name,auth = (username, password))
            x = f.content
            soup = BeautifulSoup(x, "lxml")
            i = str(soup.findAll('a', attrs = {'href': re.compile('(?i)(V10\d)')})[0])
            match = re.search(r'_V10\w+', i)
            PROBAV_Version = match.group()

            # Define some output names
            PROBAV_filename = "PROBAV_%s_%s_X%02dY%02d_%s_100M%s.HDF5" %(type_PROBAV.split('_')[-1], type_PROBAV.split('_')[0], X_tile, Y_tile, PROBAV_date, PROBAV_Version)
            PROBAV_name = 'PV_%s_%s-%s_100M%s' %(type_PROBAV.split('_')[-1], type_PROBAV.split('_')[0], PROBAV_date, PROBAV_Version)
            PROBAV_filename_tiff = os.path.join(output_folder_temp, "PROBAV_%s_%s_X%02dY%02d_%s_100M%s_%s.tif" %(type_PROBAV.split('_')[-1], type_PROBAV.split('_')[0], X_tile, Y_tile, PROBAV_date, PROBAV_Version, type_Band))
             
            # Output 
            output_file_PROBAV = os.path.join(output_folder_temp, PROBAV_filename)
            if not os.path.exists(PROBAV_filename_tiff):
                if not os.path.exists(output_file_PROBAV):
                    URL = "https://www.vito-eodata.be/PDF/datapool/Free_Data/PROBA-V_100m/%s_%s_100_m_C1/%s/%s/%s/%s/%s" % (type_PROBAV.split('_')[-1], type_PROBAV.split('_')[0], year_data, month_data, day_data,
                                                          PROBAV_name, PROBAV_filename)
                    # Download the ASCAT data
                    try:
                        y = requests.get(URL, auth=HTTPBasicAuth(username, password))
                    except:
                        from requests.packages.urllib3.exceptions import InsecureRequestWarning
                        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
                
                        y = requests.get(URL, auth=(username, password), verify=False)
                    
                    # Write the file in system
                    try:                        
                        z = open(output_file_PROBAV, 'wb')
                        z.write(y.content)
                        z.close()
                    except:
                        print("%s is not available" %output_file_PROBAV)
                    
                # Convert hdf5 into tiff    
                try:
                    # output PROBA-V tiff
                    Band_number = Band_numbers[type_Band]
                    if not type_Band == "SM":
                        scaling_factor = 0.005
                    else:
                        scaling_factor = 1.0  
                    
                    # Define the x and y spacing
                    g = gdal.Open(output_file_PROBAV, gdal.GA_ReadOnly)
                    Meta_data = g.GetMetadata()
                    Lat_Top = float(Meta_data['LEVEL3_GEOMETRY_TOP_RIGHT_LATITUDE'])
                    Lon_Left = float(Meta_data['LEVEL3_GEOMETRY_BOTTOM_LEFT_LONGITUDE'])
                    Pixel_size = float((Meta_data['LEVEL3_RADIOMETRY_BLUE_TOC_MAPPING']).split(' ')[-3]) 
                    
                    # Define the georeference of the HDF5 file
                    geo_out = [Lon_Left-0.5*Pixel_size, Pixel_size, 0, Lat_Top+0.5*Pixel_size, 0, -Pixel_size] 
        
                    # Convert hdf5 to tiff
                    DC.Convert_hdf5_to_tiff(output_file_PROBAV, PROBAV_filename_tiff, Band_number, scaling_factor, geo_out)
                  
                    # Define all the good outputs
                    output_files_PROBAV = np.append(output_files_PROBAV, PROBAV_filename_tiff)
                    
                except:
                    print("%s is not created, and replaced by fake dataset" %PROBAV_filename_tiff)
 
                    # Create empty dataset with nan values
                    Empty_Data = np.ones([10080, 10080]) * np.nan
                    Size_Pixel_PROBAV = 0.000992063492063
                    x_min = X_tile * 10 - 0.5 * Size_Pixel_PROBAV - 180
                    y_max =  Y_tile * -10 + 0.5 * Size_Pixel_PROBAV + 75      
                    Geo_out_fake = tuple([x_min, Size_Pixel_PROBAV, 0, y_max, 0, -Size_Pixel_PROBAV])
                    
                    # Save the fake layer
                    DC.Save_as_tiff(PROBAV_filename_tiff, Empty_Data, Geo_out_fake, "WGS84")
                    
            else:
                # Define all the good outputs
                output_files_PROBAV = np.append(output_files_PROBAV, PROBAV_filename_tiff)
                                  
    return(output_files_PROBAV)

def Create_Array(output_files_PROBAV, latlim, lonlim):
    
    # Define the starting values for lat and lon
    x_uniques = []
    y_uniques = []  
    x_min = [180]
    y_max = [-90]    
    
    # Loop over the output file of PROBA-V to define the extent
    for output_file_PROBAV in output_files_PROBAV:
        dest = gdal.Open(output_file_PROBAV)
        geo_one = dest.GetGeoTransform()
        size_x = dest.RasterXSize
        size_y = dest.RasterYSize
        
        # Define the latitude and longitude per pixel
        xs = np.arange(0,size_x) * geo_one[1] + geo_one[0]
        ys = np.arange(0,size_y) * geo_one[5] + geo_one[3]    
        
        # Define the right pixels within the area
        xs_bool = [np.logical_and(xs > lonlim[0], xs < lonlim[1])][0]
        ys_bool = [np.logical_and(ys > latlim[0], ys < latlim[1])][0]

        # Define the min and max pixels within the area
        x_min = np.minimum(x_min, np.min(xs[xs_bool]))
        y_max = np.maximum(y_max, np.max(ys[ys_bool]))
        
        # Add the latitude and longitude in the end pixels
        x_uniques = np.unique(np.append(x_uniques, xs[xs_bool]))
        y_uniques = np.unique(np.append(y_uniques, ys[ys_bool]))   
       
    # Create end dataset    
    Data = np.ones([len(y_uniques), len(x_uniques)]) * np.nan  
    
    # Loop over input files to fill in the end dataset
    for output_file_PROBAV in output_files_PROBAV:
        
        # Open dataset
        dest = gdal.Open(output_file_PROBAV)
        geo_one = dest.GetGeoTransform()
        
        # Define area
        xs = np.arange(0,size_x) * geo_one[1] + geo_one[0]
        ys = np.arange(0,size_y) * geo_one[5] + geo_one[3]    
        xs_bool = [np.logical_and(xs >= lonlim[0], xs <= lonlim[1])][0]
        ys_bool = [np.logical_and(ys >= latlim[0], ys <= latlim[1])][0]
        yxs_bool = xs_bool * ys_bool[:, None]   
        
        # Define area for end dataset
        xs_end = np.sort(x_uniques)
        ys_end = np.flipud(np.sort(y_uniques)) 
        xs_bool_end = [np.logical_and(xs_end >= np.maximum(lonlim[0],xs[0]), xs_end <= np.minimum(lonlim[1],xs[-1]))][0]
        ys_bool_end = [np.logical_and(ys_end >= np.maximum(latlim[0],ys[-1]), ys_end <= np.minimum(latlim[1],ys[0]))][0]
        yxs_bool_end = xs_bool_end * ys_bool_end[:, None] 
        
        # Open dataset 
        Array_one = dest.GetRasterBand(1).ReadAsArray()
        
        # Add dataset to end dataset
        Data[np.where(yxs_bool_end==True)] = Array_one[np.where(yxs_bool==True)]
        
        # remove the dest
        dest = None
        
    # Create end geo transform    
    Geo_out = tuple([x_min[0], geo_one[1], 0, y_max[0], 0, geo_one[5]])    
    
    return(Data, Geo_out)


# -*- coding: utf-8 -*-
import sys
from watertools.Collect.MERRA.DataAccess import DownloadData


def main(Dir, Years, latlim, lonlim, Waitbar = 1):
    """
    This function downloads MERRA temperature for the defined years

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['t2m', 'v2m'] 
    Years -- [2009, 2010]
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """
    for Year in Years:

        Startdate = "%s-01-01" %(Year)
        Enddate = "%s-12-31" %(Year)
        
        if Waitbar == 1:
            print('\nDownloading yearly MERRA t2m amplitude data for the period %s till %s' %(Startdate, Enddate))

        # Download data
        DownloadData(Dir, 't2m', Startdate, Enddate, latlim, lonlim, "yearly", Period = '', Waitbar = 1)

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Created on Thu Nov 22 11:34:43 2018

@author: tih
"""
import numpy as np
import pandas as pd
import glob
import os
from osgeo import gdal, osr
from pyproj import Proj, transform

#input_folders = [r"G:\SEBAL_Tadla\Daily_SEBAL\Dataset_out\PreSEBAL_SEBAL_out\NDVI",r"G:\SEBAL_Tadla\Daily_SEBAL\Dataset_out\NDVI_daily",""]
#input_formats = ["*{yyyy}{mm:02d}{dd:02d}.tif", "*{yyyy}_{doy}.tif"]
#Coordinate = [-6.80689, 32.46206]
#Startdate = "2016-10-01"
#Enddate = "2017-09-30"

def Visualize_Graph(input_folders, input_formats, Coordinate, Startdate, Enddate, freq = "D", vmin = None, vmax = None):

    Dates = pd.date_range(Startdate, Enddate, freq = freq)
    
    Datasets_end = np.ones([len(Dates), len(input_folders)+1]) * np.nan
    
    for j in range(0,len(input_folders)):
    
        input_folder = input_folders[j]  
        input_format = input_formats[j]
        Dataset = Get_Dataset_Point(input_folder, input_format, Dates, Coordinate)    
        
        if j == 0:
            
            Datasets_end[:,0:2] = Dataset
            
        else:
            Datasets_end[:,j + 1] = Dataset[:,1]
            
    Datasets_end[Datasets_end==-9999]=np.nan
    
    import matplotlib.pyplot as plt
    
    colors = ['red','blue','green','black','yellow']
    
    for k in range(0,len(input_folders)):
        plt.scatter(Dates, Datasets_end[:,k+1], s =10, c = colors[k])
    plt.ylim(vmin, vmax)
    plt.legend(input_folders, loc='upper center',bbox_to_anchor=(0.5, -0.1))
    plt.show()  
    
    return(plt)



def Get_Dataset_Point(input_folder, input_format, Dates, Coordinate):
      
    Dataset = np.ones([len(Dates), 2]) * np.nan
    i = 0
    
    for Date in Dates:

        Day = Date.day
        Month = Date.month
        Year = Date.year
        Hour = Date.hour
        Minute = Date.minute
        DOY = Date.dayofyear
        Dataset[i,0] = '%d%02d%02d%02d%02d' %(Year, Month, Day, Hour, Minute)
        os.chdir(input_folder)
        filename = glob.glob(input_format.format(yyyy=Year, mm=Month, dd=Day, doy = DOY, HH = Hour, MM=Minute))
    
        if len(filename) > 0: 
            try:
                filename = filename[0]
                
                if not'yID' in locals():
                    yID, xID = Get_Row_Column_CoordinateWGS(filename, Coordinate)
                
                if (np.isnan(yID) or np.isnan(xID)):
                    Value = np.nan
                else:
                    dest = gdal.Open(filename)
                    Array = dest.GetRasterBand(1).ReadAsArray()
                    Value = Array[yID, xID]
                
                Dataset[i,1] = Value
            except:   
                Dataset[i,1] = np.nan
        i += 1

    del yID, xID
    
    return(Dataset)

    
def Get_Row_Column_CoordinateWGS(filename, Coordinate):
             
    dest = gdal.Open(filename)
    geo_out = dest.GetGeoTransform()
    size_x = dest.RasterXSize
    size_y = dest.RasterYSize
    ulx = np.arange(0,size_x) * geo_out[1] + geo_out[0] + 0.5 *geo_out[1]
    uly = np.arange(0,size_y) * geo_out[5] + geo_out[3] + 0.5 *geo_out[5]   
    ulxx = np.zeros([len(uly),len(ulx)])
    ulyy = np.zeros([len(uly),len(ulx)])
    ulxx[:,:]=ulx[None,:]
    ulyy[:,:]=uly[:,None]
    
    ulx_flat = ulxx.flatten()
    uly_flat = ulyy.flatten()
    
    proj = osr.SpatialReference(wkt=dest.GetProjection())
    EPSG = int(proj.GetAttrValue('AUTHORITY',1))
    
    if EPSG != 4326:   
        inProj = Proj(init='epsg:%d' %EPSG)
        outProj = Proj(init='epsg:4326')
        ulx_flat, uly_flat = transform(inProj,outProj,ulx_flat, uly_flat)

    ulx_wgs = ulx_flat.reshape([len(uly),len(ulx)])              
    uly_wgs = uly_flat.reshape([len(uly),len(ulx)])            
           
    ulx_wgs_dis = np.abs(ulx_wgs - Coordinate[0])
    uly_wgs_dis = np.abs(uly_wgs - Coordinate[1])
    tot_dis = ulx_wgs_dis + uly_wgs_dis
    if np.nanmin(tot_dis) > (4 * (np.abs(ulx_wgs[0,0]-ulx_wgs[1,1]) + np.abs(uly_wgs[0,0]-uly_wgs[1,1]))):
        yID, xID = [np.nan, np.nan]
    else:
        yID, xID = np.argwhere(tot_dis == np.nanmin(tot_dis))[0]
    
    return(yID, xID)     
    
 
    
'''   
 
input_folders = [r"G:\SEBAL_Tadla\Daily_SEBAL\Dataset_out\Albedo_daily",r"G:\SEBAL_Tadla\Daily_SEBAL\Dataset_out\PreSEBAL_out\ALBEDO_MODIS_test\Albedo_new","G:\SEBAL_Tadla\Daily_SEBAL\Dataset_out\PreSEBAL_SEBAL_out\Albedo"]
input_formats = ["*{yyyy}_{doy}.tif", "*{yyyy}{mm:02d}{dd:02d}.tif", "*{yyyy}{mm:02d}{dd:02d}.tif"]
Coordinate = [-6.80689, 32.46206]
Startdate = "2016-10-01"
Enddate = "2017-09-30"

plt = Visualize_Graph(input_folders, input_formats, Coordinate, Startdate, Enddate)   
    
'''
    
    
    
    
    
    # Open data and get the value
    
    
    
    
    
    
    
    
    
    
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    




# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Products


Description:
This module contains scripts used to create WA+ products (data directly from web).

Products                      Dates                             Password                        
ETens (monthly)               2003/01/01-2014/12/31             WA+ FTP
ETref (daily)                 2000/01/01-now                    NASA
ETref (monthly)               2000/01/01-now                    NASA

Examples:
from watertools import Products
help(Products)
dir(Products)
"""

from watertools.Products import ETref
from watertools.Products import ETens
from watertools.Products import SoilGrids
from watertools.Products import RH

__all__ = ['ETref', 'ETens', 'SoilGrids', 'RH']

__version__ = '0.1'


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD17

Description:
This module downloads MOD17 GPP data from
http://e4ftl01.cr.usgs.gov/. Use the MOD17.GPP_8daily function to
download and create 8 daily GPP images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from watertools.Collect import MOD17
MOD17.GPP_8daily(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
MOD17.NPP_yearly(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .GPP_8daily import main as GPP_8daily
from .NPP_yearly import main as NPP_yearly

__all__ = ['GPP_8daily', 'NPP_yearly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
         UNESCO-IHE 2016
Contact: t.hessels@unesco-ihe.org
Repository: https://github.com/wateraccounting/wa
Module: Collect/TRMM
"""

import numpy as np
import os
import pandas as pd
import requests
import calendar
from joblib import Parallel, delayed

import watertools
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores, TimeCase):
    """
    This function downloads TRMM daily or monthly data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine. It can be 'False'
             to avoid using parallel computing routines.
    TimeCase -- String equal to 'daily' or 'monthly'
    Waitbar -- 1 (Default) will print a waitbar
    """
    # String Parameters
    if TimeCase == 'daily':
        TimeFreq = 'D'
        output_folder = os.path.join(Dir, 'Precipitation', 'TRMM', 'Daily')
    elif TimeCase == 'monthly':
        TimeFreq = 'MS'
        output_folder = os.path.join(Dir, 'Precipitation', 'TRMM', 'Monthly')
    else:
        raise KeyError("The input time interval is not supported")

	# Make directory
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

	# Check variables
    if not Startdate:
        Startdate = pd.Timestamp('1998-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('Now')
    Dates = pd.date_range(Startdate,  Enddate, freq=TimeFreq)

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    if latlim[0] < -50 or latlim[1] > 50:
        print('Latitude above 50N or below 50S is not possible.'
               ' Value set to maximum')
        latlim[0] = np.max(latlim[0], -50)
        latlim[1] = np.min(lonlim[1], 50)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W.'
               ' Now value is set to maximum')
        lonlim[0] = np.max(latlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Define IDs
    yID = np.int16(np.array([np.ceil((latlim[0] + 50)*4),
                                   np.floor((latlim[1] + 50)*4)]))
    xID = np.int16(np.array([np.floor((lonlim[0])*4),
                             np.ceil((lonlim[1])*4)]) + 720)

    # Pass variables to parallel function and run
    args = [output_folder, TimeCase, xID, yID, lonlim, latlim]

    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)

    return results


def RetrieveData(Date, args):
    """
    This function retrieves TRMM data for a given date from the
    ftp://disc2.nascom.nasa.gov server.

    Keyword arguments:
    Date -- 'yyyy-mm-dd'
    args -- A list of parameters defined in the DownloadData function.
    """
    # Argument
    [output_folder, TimeCase, xID, yID, lonlim, latlim] = args

    year = Date.year
    month= Date.month
    day = Date.day

    username, password = watertools.Functions.Random.Get_Username_PWD.GET('NASA')

    # Create https
    if TimeCase == 'daily':
        URL = 'https://disc2.gesdisc.eosdis.nasa.gov/opendap/TRMM_L3/TRMM_3B42_Daily.7/%d/%02d/3B42_Daily.%d%02d%02d.7.nc4.ascii?precipitation[%d:1:%d][%d:1:%d]'  %(year, month, year, month, day, xID[0], xID[1]-1, yID[0], yID[1]-1)
        DirFile = os.path.join(output_folder, "P_TRMM3B42.V7_mm-day-1_daily_%d.%02d.%02d.tif" %(year, month, day))
        Scaling = 1

    if TimeCase == 'monthly':
        if Date >= pd.Timestamp('2010-10-01'):
            URL = 'https://disc2.gesdisc.eosdis.nasa.gov/opendap/TRMM_L3/TRMM_3B43.7/%d/3B43.%d%02d01.7.HDF.ascii?precipitation[%d:1:%d][%d:1:%d]'  %(year, year, month, xID[0], xID[1]-1, yID[0], yID[1]-1)

        else:
            URL = 'https://disc2.gesdisc.eosdis.nasa.gov/opendap/TRMM_L3/TRMM_3B43.7/%d/3B43.%d%02d01.7A.HDF.ascii?precipitation[%d:1:%d][%d:1:%d]'  %(year, year, month, xID[0], xID[1]-1, yID[0], yID[1]-1)

        Scaling = calendar.monthrange(year,month)[1] * 24
        DirFile = os.path.join(output_folder, "P_TRMM3B43.V7_mm-month-1_monthly_%d.%02d.01.tif" %(year, month))

    if not os.path.isfile(DirFile):
        dataset = requests.get(URL, allow_redirects=False,stream = True)
        try:
            get_dataset = requests.get(dataset.headers['location'], auth = (username,password),stream = True)
        except:
            from requests.packages.urllib3.exceptions import InsecureRequestWarning
            requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
            get_dataset  = requests.get(dataset.headers['location'], auth = (username, password), verify = False)

        # download data (first save as text file)
        pathtext = os.path.join(output_folder,'temp.txt')
        z = open(pathtext,'wb')
        z.write(get_dataset.content)
        z.close()

        # Open text file and remove header and footer
        data_start = np.genfromtxt(pathtext,dtype = float,skip_header = 1,delimiter=',')
        data = data_start[:,1:] * Scaling
        data[data < 0] = -9999
        data = data.transpose()
        data = np.flipud(data)

        # Delete .txt file
        os.remove(pathtext)

        # Make geotiff file
        geo = [lonlim[0], 0.25, 0, latlim[1], 0, -0.25]
        DC.Save_as_tiff(name=DirFile, data=data, geo=geo, projection="WGS84")

    return True



# -*- coding: utf-8 -*-
"""
WaterSat
author: Tim Martijn Hessels
Created on Tue Feb 19 07:10:44 2019
"""
import os
import requests
import datetime
import pandas as pd
import numpy as np
from netCDF4 import Dataset

import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Time = '', GMT_Offset = 0, Waitbar = 1, Type = "SDS"):
    
    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -90 or latlim[1] > 90:
        print('Latitude above 90N or below 90S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0], -90)
        latlim[1] = np.min(latlim[1], 90)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)
    
    
    output_folder = os.path.join(Dir, "MSGCPP", Type, "15min")
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    
    if isinstance(Enddate, str):
        Enddate = datetime.datetime(int(Enddate.split('-')[0]), int(Enddate.split('-')[1]), int(Enddate.split('-')[2]), 23, 59)
    else:    
        Enddate = datetime.datetime(Enddate.year, Enddate.month, Enddate.day, 23, 59)
    
    if Time == '':
        Dates = pd.date_range(Startdate, Enddate, freq = "15min") - datetime.timedelta(hours=GMT_Offset)
    else:
        Dates = pd.date_range(Startdate, Enddate, freq = "D") - datetime.timedelta(hours=GMT_Offset)
    
    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
    
    # Loop over dates
    for Date in Dates:
        
        if Time != '':
            Hour = int(Time.split(':')[0])
            Minute = int(Time.split(':')[1])        
            Date = datetime.datetime(Date.year, Date.month, Date.day, Hour, Minute)
            
        if Type == 'SDS':
            filename_out = os.path.join(output_folder, "SDS_MSGCPP_W-m-2_15min_%d.%02d.%02d_H%02d.M%02d.tif" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            filename_out_nc = os.path.join(output_folder, "SDS_MSGCPP_W-m-2_15min_%d.%02d.%02d_H%02d.M%02d.nc" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            parameter = 'sds'

        if Type == 'lwe_precipitation_rate':
            filename_out = os.path.join(output_folder, "Precipitation_MSGCPP_mm-h-1_15min_%d.%02d.%02d_H%02d.M%02d.tif" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            filename_out_nc = os.path.join(output_folder, "Precipitation_MSGCPP_mm-h-1_15min_%d.%02d.%02d_H%02d.M%02d.nc" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            parameter = 'precip'
            
        if Type == 'lwe_precipitation_rate_ir':
            filename_out = os.path.join(output_folder, "PrecipitationIR_MSGCPP_mm-h-1_15min_%d.%02d.%02d_H%02d.M%02d.tif" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            filename_out_nc = os.path.join(output_folder, "PrecipitationIR_MSGCPP_mm-h-1_15min_%d.%02d.%02d_H%02d.M%02d.nc" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            parameter = 'precip_ir'
            
        if Type == 'Cloud':
            filename_out = os.path.join(output_folder, "Cloud_MSGCPP_-_15min_%d.%02d.%02d_H%02d.M%02d.tif" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            filename_out_nc = os.path.join(output_folder, "Cloud_MSGCPP_-_15min_%d.%02d.%02d_H%02d.M%02d.nc" %(Date.year, Date.month, Date.day, Date.hour, Date.minute))
            parameter = 'cldmask'

        if not os.path.exists(filename_out):
        
            # define url
            if Type == 'SDS':
                url = r"https://msgcpp-adaguc.knmi.nl/adaguc-server/?dataset=msgrt&service=wcs&request=getcoverage&coverage=surface_downwelling_shortwave_flux_in_air&FORMAT=NetCDF4&CRS=EPSG%%3A4326&BBOX=%s,%s,%s,%s&RESX=0.04310344827586207&RESY=0.04418103448275862&time=%d-%02d-%02dT%02d%%3A%02d%%3A00Z" %(lonlim[0],latlim[0], lonlim[1], latlim[1], Date.year, Date.month, Date.day, Date.hour, Date.minute)
            if Type == 'lwe_precipitation_rate':
                url = r"https://msgcpp-adaguc.knmi.nl/adaguc-server/?dataset=msgrt&service=wcs&request=getcoverage&coverage=lwe_precipitation_rate&FORMAT=NetCDF4&CRS=EPSG%%3A4326&BBOX=%s,%s,%s,%s&RESX=0.04310344827586207&RESY=0.04418103448275862&time=%d-%02d-%02dT%02d%%3A%02d%%3A00Z" %(lonlim[0],latlim[0], lonlim[1], latlim[1], Date.year, Date.month, Date.day, Date.hour, Date.minute)
            if Type == 'lwe_precipitation_rate_ir':
                url = r"https://msgcpp-adaguc.knmi.nl/adaguc-server/?dataset=msgrt&service=wcs&request=getcoverage&coverage=lwe_precipitation_rate_ir&FORMAT=NetCDF4&CRS=EPSG%%3A4326&BBOX=%s,%s,%s,%s&RESX=0.04310344827586207&RESY=0.04418103448275862&time=%d-%02d-%02dT%02d%%3A%02d%%3A00Z" %(lonlim[0],latlim[0], lonlim[1], latlim[1], Date.year, Date.month, Date.day, Date.hour, Date.minute)
            if Type == 'Cloud':
                url = r"https://msgcpp-adaguc.knmi.nl/adaguc-server/?dataset=msgrt&service=wcs&request=getcoverage&coverage=cloud_area_fraction&FORMAT=NetCDF4&CRS=EPSG%%3A4326&BBOX=%s,%s,%s,%s&RESX=0.04310344827586207&RESY=0.04418103448275862&time=%d-%02d-%02dT%02d%%3A%02d%%3A00Z" %(lonlim[0],latlim[0], lonlim[1], latlim[1], Date.year, Date.month, Date.day, Date.hour, Date.minute)

            print(url)
            success = 0
            attempts = 0
            while success == 0 and attempts<4:
                try:
                    request = requests.get(url, timeout=10, stream=True)
                    with open(filename_out_nc, 'wb') as fh:
                        # Walk through the request response in chunks of 1024 * 1024 bytes, so 1MiB
                        for chunk in request.iter_content(1024 * 1024):
                            # Write the chunk to the file
                            fh.write(chunk)
                            # Optionally we can check here if the download is taking too long                    
                    #urllib.request.urlretrieve(url, filename=)
                    statinfo = os.stat(filename_out_nc)
                    if statinfo.st_size > 300:
                        success = 1
                        
                    else:
                        attempts +=1
                        
                        
                except Exception as e:
                    print("ERROR: %s" %e)     
                    attempts +=1

            if statinfo.st_size < 300:
                os.remove(filename_out_nc)
                
                # maak hier nep file met nans
                #0.04310344827586207&RESY=0.04418103448275862
            else:
                
                fh = Dataset(filename_out_nc)
                Array = np.array(fh[parameter][:,:].data.squeeze())
                Array[Array<0] = np.nan
                x = fh["x"][:]               
                y = fh["y"][:]            
                x_size = (np.nanmax(x) - np.nanmin(x))/(len(x)-1)
                y_size = (np.nanmax(y) - np.nanmin(y))/(len(y)-1)
                lon_start = np.nanmin(x) - x_size/2
                lat_start = np.nanmax(y) + y_size/2
                proj = 4326
                geo = tuple([lon_start, x_size, 0, lat_start, 0, -y_size])
                
                DC.Save_as_tiff(filename_out, Array, geo, proj)
                
                try:
                    fh.close()
                    os.remove(filename_out_nc)
                except Exception as e:
                    print("ERROR removing %s" %filename_out_nc)
                    print("MESSAGE: %s" %e)               
        
        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
    
        


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD11

Description:
This module downloads MOD11 LST data from
http://e4ftl01.cr.usgs.gov/. Use the MOD11.LST_8daily function to
download and create 8 daily LST images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from watertools.Collect import MOD11
MOD11.LST_8daily(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-30',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .LST_8daily import main as LST_8daily
from .LST_daily import main as LST_daily

__all__ = ['LST_8daily', 'LST_daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect/DEM
"""

#General modules
import os
import sys

# Water Accounting modules
from watertools.Collect.DEM.DataAccess_Hydro import DownloadData

def main(Dir, latlim, lonlim, resolution = '15s', Waitbar = 1):
    """
    Downloads HydroSHED flow accumulation data from http://www.hydrosheds.org/download/

    this data includes a Digital Elevation Model Accumulation Direction
    The spatial resolution is 90m (3s) or 450m (15s) or 900m (30s)

    The following keyword arguments are needed:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    resolution -- '3s' (default) or '15s' or '30s'
    """

    # Create directory if not exists for the output
    output_folder = os.path.join(Dir, 'HydroSHED', 'ACC')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Define the output map and create this if not exists  # Define the output map and create this if not exists
    nameEnd = os.path.join(Dir, 'HydroSHED', 'ACC', 'ACC_HydroShed_-_%s.tif' %resolution)
    parameter = "acc_%s" %resolution

    if not os.path.exists(nameEnd):

        # Create Waitbar
        if Waitbar == 1:
            print('\nDownload HydroSHED Drainage Accumulation map with a resolution of %s' %resolution)
            import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
            total_amount = 1
            amount = 0
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

        # Download and process the data
        DownloadData(output_folder, latlim, lonlim, parameter, resolution)

        if Waitbar == 1:
            amount = 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    else:
        if Waitbar == 1:
            print("\nHydroSHED Drainage Accumulation (%s) already exists in output folder" %resolution)

if __name__ == '__main__':
    main(sys.argv)




# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: watertools
Module: watertools/Collect/MSGCPP

Description:
This module downloads daily or instantanious MSGCPP data from
http://msgcpp.knmi.nl/portal/ server. The MSGCPP data is available since 2017-01-01 till the present.
The datasets will be stored in the user defined outputfolder in GEOTIFF format.


Examples:
from watertools.Collect import MSGCPP
MSGCPP.SDS(Dir='C:/Temp/', Startdate='1999-02-01', Enddate='1999-02-03',
             latlim=[-10, 30], lonlim=[-20, 120], Time = "13:45")		 
"""

from .SDS import main as SDS
from .Precipitation import main as Precipitation
from .Cloud import main as Cloud

__all__ = ['SDS', "Precipitation", "Cloud"]

__version__ = '0.1'



# -*- coding: utf-8 -*-
import sys
from watertools.Collect.TWC.DataAccess import DownloadData


def main(Dir, latlim=[-60, 84], lonlim=[-180, 180]):

    """
    This function downloads gray water footprint data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax] (values must be between -60 and 84)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # Download data
    DownloadData(Dir, latlim, lonlim)

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: watertools
Module: watertools/Collect/MERRA

Description:
This module downloads daily or instantanious MERRA data from
https://opendap.nccs.nasa.gov/dods/GEOS-5/MERRAero server. The MERRA data is available since 1979-01-01 till the present.
The datasets will be stored in the user defined outputfolder in GEOTIFF format.


Examples:
from watertools.Collect import MERRA
MERRA.daily(Dir='C:/Temp/', Vars = ["t2m", "v2m"], Startdate='2002-02-01', Enddate='2002-02-03',
             latlim=[-10, 30], lonlim=[-20, 120])
MERRA.three_hourly(Dir='C:/Temp/', Vars =["t2m", "v2m"], Startdate='2002-02-01', Enddate='2002-02-03',
             latlim=[-10, 30], lonlim=[-20, 120], Periods = [1,2,3,4,5])			 
"""

from .hourly_MERRA2 import main as hourly_MERRA2
from .daily_MERRA2 import main as daily_MERRA2
from .daily import main as daily
from .three_hourly import main as three_hourly
from. yearly_T_Amplitude import main as yearly_T_Amplitude

__all__ = ['daily', 'three_hourly', 'yearly_T_Amplitude']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD16
"""

# import general python modules
import os
import numpy as np
import pandas as pd
from osgeo import gdal
import urllib
import re
import glob
from joblib import Parallel, delayed
from bs4 import BeautifulSoup
import datetime
import math
import sys
import requests
if sys.version_info[0] == 3:
    import urllib.request    
    import urllib.parse
if sys.version_info[0] == 2:
    import urllib2

# Water Accounting modules
import watertools
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, timestep, Waitbar, cores, hdf_library, remove_hdf):
    """
    This function downloads MOD13 16-daily data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -90 and 90)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine. It can be 'False'
             to avoid using parallel computing routines.
    Waitbar -- 1 (Default) will print a waitbar
    """

    # Check start and end date and otherwise set the date
    if not Startdate:
        Startdate = pd.Timestamp('2000-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('2014-12-31')

    # Make an array of the days of which the ET is taken
    if timestep == 'monthly':
        Dates = pd.date_range(Startdate,Enddate,freq = 'M')
        TIMESTEP = 'Monthly'
        Size_pix = 1
    elif timestep == '8-daily':
        Dates = Make_TimeStamps(Startdate,Enddate)
        TIMESTEP = '8_Daily'
        Size_pix = 2
        
    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Make directory for the MODIS ET data
    output_folder=os.path.join(Dir,'Evaporation','MOD16', TIMESTEP)
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    TilesVertical, TilesHorizontal = watertools.Collect.MOD15.DataAccess.Get_tiles_from_txt(output_folder, hdf_library, latlim, lonlim)

    # Pass variables to parallel function and run
    args = [output_folder, TilesVertical, TilesHorizontal,latlim, lonlim, timestep, hdf_library, Size_pix]
    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)

    if remove_hdf == 1:
        # Remove all .hdf files
        os.chdir(output_folder)
        files = glob.glob("*.hdf")
        for f in files:
            os.remove(os.path.join(output_folder, f))

        # Remove all .txt files
        #files = glob.glob("*.txt")
        #for f in files:
        #    os.remove(os.path.join(output_folder, f))

    return(results)

def RetrieveData(Date, args):
    """
    This function retrieves MOD16 ET data for a given date from the
    ftp://ftp.ntsg.umt.edu/ server.

    Keyword arguments:
    Date -- 'yyyy-mm-dd'
    args -- A list of parameters defined in the DownloadData function.
    """
    # Argument
    [output_folder, TilesVertical, TilesHorizontal,latlim, lonlim, timestep, hdf_library, Size_pix] = args

    if timestep == 'monthly':
         ETfileName = os.path.join(output_folder, 'ET_MOD16A2_mm-month-1_monthly_'+Date.strftime('%Y')+'.' + Date.strftime('%m')+'.01.tif')
    elif timestep == '8-daily':
         ETfileName = os.path.join(output_folder, 'ET_MOD16A2_mm-8days-1_8-daily_'+Date.strftime('%Y') + '.' + Date.strftime('%m') + '.' + Date.strftime('%d') + '.tif')


    if not os.path.exists(ETfileName):

        # Collect the data from the MODIS webpage and returns the data and lat and long in meters of those tiles
        try:
            Collect_data(TilesHorizontal,TilesVertical,Date,output_folder, timestep, hdf_library, Size_pix)
        except:
            print("Was not able to download the file")
        try:
            # Define the output name of the collect data function
            name_collect = os.path.join(output_folder, 'Merged.tif')
        
            # Reproject the MODIS product to epsg_to
            epsg_to ='4326'
            name_reprojected = RC.reproject_MODIS(name_collect, epsg_to)
        
            # Clip the data to the users extend
            data, geo, proj = RC.clip_data(name_reprojected, latlim, lonlim)
        
        
            DC.Save_as_tiff(name=ETfileName, data=data, geo=geo, projection='WGS84')
        
            # remove the side products
            os.remove(os.path.join(output_folder, name_collect))
            os.remove(os.path.join(output_folder, name_reprojected))
        except:
            print("Failed for date: %s" %Date)
            
    return()



def Collect_data(TilesHorizontal,TilesVertical,Date,output_folder,timestep, hdf_library, Size_pix):
    '''
    This function downloads all the needed MODIS tiles from ftp.ntsg.umt.edu/pub/MODIS/NTSG_Products/MOD16/MOD16A2_MONTHLY.MERRA_GMAO_1kmALB/ as a hdf file.

    Keywords arguments:
    TilesHorizontal -- [TileMin,TileMax] max and min horizontal tile number
    TilesVertical -- [TileMin,TileMax] max and min vertical tile number
    Date -- 'yyyy-mm-dd'
    output_folder -- 'C:/file/to/path/'
    '''

    # Make a new tile for the data
    sizeX=int(TilesHorizontal[1]-TilesHorizontal[0]+1)*1200 * Size_pix
    sizeY=int(TilesVertical[1]-TilesVertical[0]+1)*1200 * Size_pix
    DataTot=np.ones((sizeY,sizeX))* -9999

    # Make a new tile for the lat and long info
    LatMet=np.zeros((sizeY))
    LongMet=np.zeros((sizeX))

    # Create the Lat and Long of the MODIS tile in meters
    for Vertical in range(int(TilesVertical[0]),int(TilesVertical[1])+1):
        Distance=926.625 / Size_pix # resolution of a MODIS pixel in meter
        countY=int((TilesVertical[1]-TilesVertical[0]+1)-(Vertical-TilesVertical[0]))
        LatMet[int((countY-1)*1200 * Size_pix):int((countY)*1200 * Size_pix)]=np.linspace(((8-Vertical)*1200 * Size_pix+0.5)*Distance,((8-Vertical)*1200 * Size_pix + 1200 * Size_pix - 0.5)*Distance,1200 * Size_pix)

        for Horizontal in range(int(TilesHorizontal[0]),int(TilesHorizontal[1])+1):
            countX=int(Horizontal-TilesHorizontal[0]+1)
            LongMet[int((countX-1)*1200 * Size_pix):int((countX)*1200 * Size_pix)]=np.linspace(((Horizontal-18)*1200 * Size_pix+0.5)*Distance,((Horizontal-18)*1200+1200 * Size_pix - 0.5)*Distance,1200* Size_pix)

            # Set the download to zero again
            downloaded = 0

            if hdf_library is not None:
                os.chdir(hdf_library)
                hdf_name = glob.glob("MOD16A2.A%s%03s.h%02dv%02d.*" %(Date.strftime('%Y'), Date.strftime('%j'), Horizontal, Vertical))

                if len(hdf_name) == 1:
                    hdf_file = os.path.join(hdf_library, hdf_name[0])

                    if os.path.exists(hdf_file):
                        downloaded = 1
                        data = Open_mod16_data(hdf_file)
                        countYdata=(TilesVertical[1]-TilesVertical[0]+2)-countY
                        DataTot[(int(countYdata)-1)*1200 * Size_pix:int(countYdata)*1200 * Size_pix,(int(countX)-1)*1200* Size_pix:int(countX)*1200 * Size_pix]=data*0.1

            while downloaded == 0:

                # Download the MODIS FPAR data
                if timestep == 'monthly':
                    url = 'http://files.ntsg.umt.edu/data/NTSG_Products/MOD16/MOD16A2_MONTHLY.MERRA_GMAO_1kmALB/Y%s/M%02s/' %(Date.strftime('%Y'), Date.strftime('%m'))

                if timestep == '8-daily':
                    url = 'https://e4ftl01.cr.usgs.gov/MOLT/MOD16A2.006/%d.%02d.%02d/' %(Date.year, Date.month, Date.day)

                try:
                    
                    # Get files on FTP server
                    if sys.version_info[0] == 3:
                        f = urllib.request.urlopen(url)
    
                    if sys.version_info[0] == 2:
                        f = urllib2.urlopen(url)
    
                    # Sum all the files on the server
                    soup = BeautifulSoup(f, "lxml")

                    for i in soup.findAll('a', attrs = {'href': re.compile('(?i)(hdf)$')}):

                        # Find the file with the wanted tile number
                        nameHDF=str(i)
                        HDF_name = nameHDF.split('>')[-2][:-3]
                        Hfile=HDF_name[18:20]
                        Vfile=HDF_name[21:23]
                        if int(Vfile) is int(Vertical) and int(Hfile) is int(Horizontal):
                            
                            HTTP_name = url + HDF_name
                            output_name = os.path.join(output_folder, HDF_name)

                            if timestep == 'monthly':

                                if not os.path.isfile(output_name):
    
                                    while downloaded == 0:
    
                                        if sys.version_info[0] == 2:
                                            urllib.urlretrieve(HTTP_name,output_name)
                                        if sys.version_info[0] == 3:
                                            urllib.request.urlretrieve(HTTP_name,output_name)
    
                                        statinfo = os.stat(output_name)
                                        # Say that download was succesfull
                                        if int(statinfo.st_size) > 1000:
                                           downloaded = 1
                                           
                            if timestep == '8-daily':
                                            
            		                 # Reset the begin parameters for downloading
                                N=0
                                username, password = watertools.Functions.Random.Get_Username_PWD.GET('NASA')
              
                                # if not downloaded try to download file
                                while downloaded == 0:
            
                                    try:# open http and download whole .hdf
                                        file_name = os.path.join(output_folder,HTTP_name.split('/')[-1])
                                        if os.path.isfile(output_name):
                                            downloaded = 1
                                        else:
                                            x = requests.get(HTTP_name, allow_redirects = False)
                                            try:
                                                y = requests.get(x.headers['location'], auth = (username, password))
                                            except:
                                                from requests.packages.urllib3.exceptions import InsecureRequestWarning
                                                requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
            
                                                y = requests.get(x.headers['location'], auth = (username, password), verify = False)
  
                                            z = open(output_name, 'wb')
                                            z.write(y.content)
                                            z.close()
                                            statinfo = os.stat(file_name)
                                            # Say that download was succesfull
                                            if int(statinfo.st_size) > 1000:
                                                 downloaded = 1
            
                                    # If download was not succesfull
                                    except:
            
                                        # Try another time
                                        N = N + 1
            
            						      # Stop trying after 10 times
                                    if N == 10:
                                        print('Data from ' + Date.strftime('%Y-%m-%d') + ' is not available')
                                        downloaded = 1                                

                            # Open .hdf only band with ET and collect all tiles to one array
                            data = Open_mod16_data(output_name)
                            countYdata=(TilesVertical[1]-TilesVertical[0]+2)-countY
                            DataTot[(int(countYdata)-1)*1200 * Size_pix:int(countYdata)*1200 * Size_pix,(int(countX)-1)*1200* Size_pix:int(countX)*1200* Size_pix]=data*0.1
                            DataTot[DataTot>3000]=-9999
                            downloaded = 1
                            del data

                except:
                    proj='PROJCS["unnamed",GEOGCS["Unknown datum based upon the custom spheroid",DATUM["Not specified (based on custom spheroid)",SPHEROID["Custom spheroid",6371007.181,0]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433]],PROJECTION["Sinusoidal"],PARAMETER["longitude_of_center",0],PARAMETER["false_easting",0],PARAMETER["false_northing",0],UNIT["Meter",1]]'
                    data=np.ones((1200* Size_pix, 1200* Size_pix)) * (-9999)
                    countYdata=(TilesVertical[1] - TilesVertical[0] + 2) - countY
                    DataTot[(countYdata - 1) * 1200* Size_pix:countYdata * 1200* Size_pix,(countX - 1) * 1200* Size_pix:countX * 1200* Size_pix] = data * 0.1
                    downloaded = 1

	 # Make geotiff file
    name2 = os.path.join(output_folder, 'Merged.tif')
    driver = gdal.GetDriverByName("GTiff")
    dst_ds = driver.Create(name2, DataTot.shape[1], DataTot.shape[0], 1, gdal.GDT_Float32, ['COMPRESS=LZW'])
    try:
        dst_ds.SetProjection(proj)
    except:
        proj='PROJCS["unnamed",GEOGCS["Unknown datum based upon the custom spheroid",DATUM["Not specified (based on custom spheroid)",SPHEROID["Custom spheroid",6371007.181,0]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433]],PROJECTION["Sinusoidal"],PARAMETER["longitude_of_center",0],PARAMETER["false_easting",0],PARAMETER["false_northing",0],UNIT["Meter",1]]'
        x1 = (TilesHorizontal[0] - 18) * 1200 * Size_pix * Distance
        x4 = (TilesVertical[0] - 9) * 1200 * Size_pix * -1 * Distance
        geo = [x1, Distance, 0.0, x4, 0.0, -Distance]
        geo_t = tuple(geo)
        dst_ds.SetProjection(proj)

    dst_ds.GetRasterBand(1).SetNoDataValue(-9999)
    dst_ds.SetGeoTransform(geo_t)
    dst_ds.GetRasterBand(1).WriteArray(DataTot)
    dst_ds = None


    return(DataTot,LatMet,LongMet)

def Open_mod16_data(output_name):

    dataset=gdal.Open(output_name)
    sdsdict=dataset.GetMetadata('SUBDATASETS')
    sdslist =[sdsdict[k] for k in sdsdict.keys() if '_1_NAME' in k]
    sds=[]

    sds=gdal.Open(sdslist[0])
    data = np.float_(sds.ReadAsArray())
    sds = None
    return(data)


def Make_TimeStamps(Startdate,Enddate):
    '''
    This function determines all time steps of which the FPAR must be downloaded
    The time stamps are 8 daily.

    Keywords arguments:
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    '''

    # Define the DOY and year of the start day
    DOY = datetime.datetime.strptime(Startdate,'%Y-%m-%d').timetuple().tm_yday
    Year = datetime.datetime.strptime(Startdate,'%Y-%m-%d').timetuple().tm_year

    # Define the year of the end day
    YearEnd = datetime.datetime.strptime(Enddate,'%Y-%m-%d').timetuple().tm_year

    # Change the DOY of the start day into a DOY of MODIS day (8-daily) and create new startdate
    DOYstart = int(math.floor(DOY / 8.0) * 8) + 1
    DOYstart = str('%s-%s' %(DOYstart, Year))
    Day = datetime.datetime.strptime(DOYstart, '%j-%Y')
    Month = '%02d' % Day.month
    Day = '%02d' % Day.day
    Startdate = (str(Year) + '-' + str(Month) + '-' + str(Day))

    # Create the start and end data for the whole year
    YearStartDate = pd.date_range(Startdate, Enddate, freq = 'AS')
    YearEndDate = pd.date_range(Startdate, Enddate, freq = 'A')

    # Define the amount of years that are involved
    AmountOfYear = YearEnd - Year

    # If the startday is not in the same year as the enddate
    if AmountOfYear > 0:
        for i in range(0, AmountOfYear+1):
            if i == 0:
                Startdate1 = Startdate
                Enddate1 = YearEndDate[0]
                Dates = pd.date_range(Startdate1, Enddate1, freq = '8D')
            if i == AmountOfYear:
                Startdate1 = YearStartDate[-1]
                Enddate1 = Enddate
                Dates1 = pd.date_range(Startdate1, Enddate1, freq = '8D')
                Dates = Dates.union(Dates1)
            if (i != AmountOfYear and i != 0):
                Startdate1 = YearStartDate[i-AmountOfYear-1]
                Enddate1 = YearEndDate[i]
                Dates1 = pd.date_range(Startdate1, Enddate1, freq = '8D')
                Dates = Dates.union(Dates1)

    # If the startday is in the same year as the enddate
    if AmountOfYear == 0:
        Dates = pd.date_range(Startdate, Enddate, freq = '8D')

    return(Dates)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/ALEXI

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the ALEXI developers.

Description:
This module downloads ALEXI data from
ftp.wateraccounting.unesco-ihe.org. Use the ALEXI.daily function to
download and create weekly ALEXI images in Gtiff format.
The data is available between 2003-01-01 till 2015-12-31.

The output file with the name 2003.01.01 contains the total evaporation in mm
for the period of 1 January - 7 January.

Examples:
from watertools.Collect import ALEXI
ALEXI.weekly(Dir='C:/Temp/', Startdate='2003-12-01', Enddate='2004-01-20',
           latlim=[-10, 30], lonlim=[-20, -10])
"""

from .daily import main as daily
from .weekly import main as weekly
from .monthly import main as monthly

__all__ = ['daily', 'weekly', 'monthly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/GLDAS


Description:
This script automatically downloads GLDAS 2.0 or 2.1 data with a 0.25 degree
resolution for different extends based on their opendap server. The time
interval available are: three-hourly ('3hour'), daily ('day'), and monthly
('month'). A list of the variables can be printed with the command:
    GLDAS.VarInfo('daily').descriptions.keys()
Futher information of the variable can be printed with the following commands:
    GLDAS.VarInfo('daily').descriptions['evap']
    GLDAS.VarInfo('daily').units['evap']
    GLDAS.VarInfo('daily').names['evap']
    GLDAS.VarInfo('daily').factors['evap']

Examples:
from watertools.Collect import GLDAS
GLDAS.three_hourly(Dir='C:/Temp/', Vars=['qair','tair'], Startdate='2004-12-20', Enddate='2005-01-10',
                   latlim=[38, 41], lonlim=[-76, -73], Periods=[4, 5], gldas_version = '2.1')
GLDAS.daily(Dir='C:/Temp/', Vars=['qair'], Startdate='2004-12-20', Enddate='2005-01-01',
            latlim=[38, 41], lonlim=[-76, -73],
            SumMean=1, Min=1, Max=1, gldas_version = '2.1')
GLDAS.monthly(Dir='C:/TempGLDAS', Vars=['swnet'], Startdate='2004-12-20', Enddate='2005-03-10',latlim=[38, 41], lonlim=[-76, -73], gldas_version = '2.1')
"""

from .three_hourly import main as three_hourly
from .daily import main as daily
from .monthly import main as monthly
from .DataAccess import VariablesInfo as VarInfo
from .CLSM_DataAccess import VariablesInfo as CLSM_VarInfo
from .CLSM_daily import main as CLSM_daily
from .CLSM_monthly import main as CLSM_monthly
from .CLSM_three_hourly import main as CLSM_three_hourly
from .NOAH_DataAccess import VariablesInfo as NOAH_VarInfo
from .NOAH_daily import main as NOAH_daily
from .NOAH_monthly import main as NOAH_monthly
from .NOAH_three_hourly import main as NOAH_three_hourly


__all__ = ['three_hourly', 'daily', 'monthly', 'VarInfo', 'CLSM_VarInfo', 'CLSM_daily', 'CLSM_monthly', 'CLSM_three_hourly', 'NOAH_VarInfo', 'NOAH_daily', 'NOAH_monthly', 'NOAH_three_hourly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD12

Description:
This module downloads MOD12 LC data from
http://e4ftl01.cr.usgs.gov/. Use the MOD12.LC_yearly function to
download and create yearly LC images in Gtiff format.
The data is available between 2001-01-01 till 2014-01-01 .

Examples:
from watertools.Collect import MOD12
MOD17.LC_yearly(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
MOD17.LC_yearly(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .LC_yearly import main as LC_yearly

__all__ = ['LC_yearly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Function/Start
"""
# General Python modules
import numpy as np
import os
import glob
import pandas as pd
from osgeo import gdal
import calendar

def Nearest_Interpolate(Dir_in, Startdate, Enddate, Dir_out = None):
    """
    This functions calculates monthly tiff files based on the 16 daily tiff files. (will calculate the average)

    Parameters
    ----------
    Dir_in : str
        Path to the input data
    Startdate : str
        Contains the start date of the model 'yyyy-mm-dd'
    Enddate : str
        Contains the end date of the model 'yyyy-mm-dd'
    Dir_out : str
        Path to the output data, default is same as Dir_in

    """
    # import WA+ modules
    import watertools.General.data_conversions as DC
    import watertools.General.raster_conversions as RC

    # Change working directory
    os.chdir(Dir_in)

    # Find all eight daily files
    files = glob.glob('*16-daily*.tif')

    # Create array with filename and keys (DOY and year) of all the 8 daily files
    i = 0
    DOY_Year = np.zeros([len(files),3])
    for File in files:

        # Get the time characteristics from the filename
        year = File.split('.')[-4][-4:]
        month = File.split('.')[-3]
        day = File.split('.')[-2]

        # Create pandas Timestamp
        date_file = '%s-%02s-%02s' %(year, month, day)
        Datum = pd.Timestamp(date_file)

        # Get day of year
        DOY = Datum.strftime('%j')

        # Save data in array
        DOY_Year[i,0] = i
        DOY_Year[i,1] = DOY
        DOY_Year[i,2] = year

        # Loop over files
        i += 1

    # Check enddate:
    Enddate_split = Enddate.split('-')
    month_range = calendar.monthrange(int(Enddate_split[0]),int(Enddate_split[1]))[1]
    Enddate = '%d-%02d-%02d' %(int(Enddate_split[0]),int(Enddate_split[1]),month_range)

    # Check startdate:
    Startdate_split = Startdate.split('-')
    Startdate = '%d-%02d-01' %(int(Startdate_split[0]),int(Startdate_split[1]))

    # Define end and start date
    Dates = pd.date_range(Startdate, Enddate, freq='MS')
    DatesEnd = pd.date_range(Startdate, Enddate, freq='M')

    # Get array information and define projection
    geo_out, proj, size_X, size_Y = RC.Open_array_info(files[0])
    if int(proj.split('"')[-2]) == 4326:
        proj = "WGS84"

    # Get the No Data Value
    dest = gdal.Open(files[0])
    NDV = dest.GetRasterBand(1).GetNoDataValue()

    # Loop over months and create monthly tiff files
    i = 0
    for date in Dates:
        # Get Start and end DOY of the current month
        DOY_month_start = date.strftime('%j')
        DOY_month_end = DatesEnd[i].strftime('%j')

        # Search for the files that are between those DOYs
        year = date.year
        DOYs = DOY_Year[DOY_Year[:,2] == year]
        DOYs_oneMonth = DOYs[np.logical_and((DOYs[:,1] + 16) >= int(DOY_month_start), DOYs[:,1] <= int(DOY_month_end))]

        # Create empty arrays
        Monthly = np.zeros([size_Y, size_X])
        Weight_tot = np.zeros([size_Y, size_X])
        Data_one_month = np.ones([size_Y, size_X]) * np.nan

        # Loop over the files that are within the DOYs
        for EightDays in DOYs_oneMonth[:,1]:

            # Calculate the amount of days in this month of each file
            Weight = np.ones([size_Y, size_X])

            # For start of month
            if np.min(DOYs_oneMonth[:,1]) == EightDays:
                Weight =  Weight * int(EightDays + 16 - int(DOY_month_start))

            # For end of month
            elif np.max(DOYs_oneMonth[:,1]) == EightDays:
                Weight = Weight * (int(DOY_month_end) - EightDays + 1)

            # For the middle of the month
            else:
                Weight = Weight * 16

            row = DOYs_oneMonth[np.argwhere(DOYs_oneMonth[:,1]==EightDays)[0][0],:][0]

            # Open the array of current file
            input_name = os.path.join(Dir_in, files[int(row)])
            Data = RC.Open_tiff_array(input_name)

            # Remove NDV
            Weight[Data == NDV] = 0
            Data[Data == NDV] = np.nan

            # Multiply weight time data (per day)
            Data = Data * Weight

            # Calculate the total weight and data
            Weight_tot += Weight
            Monthly[~np.isnan(Data)] += Data[~np.isnan(Data)]

        # Go to next month
        i += 1

        # Calculate the average
        Data_one_month[Weight_tot != 0.] = Monthly[Weight_tot != 0.] / Weight_tot[Weight_tot != 0.]

        # Define output directory
        if Dir_out == None:
            Dir_out = Dir_in

        # Define output name
        output_name = os.path.join(Dir_out, files[int(row)].replace('16-daily', 'monthly'))
        output_name = output_name[:-9] + '%02d.01.tif' %(date.month)

        # Save tiff file
        DC.Save_as_tiff(output_name, Data_one_month, geo_out, proj)

    return












# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD10

Description:
This module downloads MOD10 SnowMask data from
https://n5eil01u.ecs.nsidc.org/MOST/MOD10A2.006. Use the MOD10.SnowMask_8daily function to
download and create 8 daily SnowMask images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from watertools.Collect import MOD10
MOD10.SnowMask_8daily(Dir='C:/Temp4/',Startdate='2003-12-01',Enddate='2003-12-20',
                                 latlim=[30,35],lonlim=[70,75])
"""

from .SnowMask_8daily import main as SnowMask_8daily
from .SnowMask_daily import main as SnowMask_daily

__all__ = ['SnowMask_8daily', 'SnowMask_daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
WaterSat
author: Tim Martijn Hessels
Created on Mon Feb 11 08:57:38 2019
"""

import sys
from watertools.Collect.L30.DataAccess import DownloadData


def main(Dir, Startdate, Enddate, S2_tiles):
                """
                Keyword arguments:
                Dir -- 'C:/file/to/path/'
                Startdate -- 'yyyy-mm-dd'
                Enddate -- 'yyyy-mm-dd'
                S2_tiles -- Tiles that you want to download
                """
                print('\nDownload L30 Harmonized data for the period %s till %s' %(Startdate, Enddate))
                
                for S2_tile in S2_tiles:
                    # Download data
                    DownloadData(Dir, Startdate, Enddate, S2_tile)

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/SoilGrids
'''

# import general python modules
import os
from osgeo import gdal
import numpy as np

# import watertools modules
from watertools.General import data_conversions as DC
from watertools.General import raster_conversions as RC

def Topsoil(Dir, latlim, lonlim):
    """
    This function calculates the topsoil saturated soil characteristic (15cm)

    Keyword arguments:
    Dir -- 'C:/' path to the map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate Theta Saturated map of the topsoil from SoilGrids')
    
    # Define parameters to define the topsoil
    SL = "sl3"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return

def Subsoil(Dir, latlim, lonlim):
    """
    This function calculates the subsoil saturated soil characteristic (100cm)

    Keyword arguments:
    Dir -- 'C:/' path to the WA map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate Theta Saturated map of the subsoil from SoilGrids')
    
	# Define parameters to define the subsoil	
    SL = "sl6"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return
	
def Calc_Property(Dir, latlim, lonlim, SL):	
	
    import watertools.Collect.SoilGrids as SG
    
    # Download needed layers
    SG.Clay_Content(Dir, latlim, lonlim, level=SL)
    SG.Organic_Carbon_Content(Dir, latlim, lonlim, level=SL)
    #SG.Bulk_Density(Dir, latlim, lonlim, level=SL)
    
    # Define path to layers
    filename_clay = os.path.join(Dir, 'SoilGrids', 'Clay_Content' ,'ClayContentMassFraction_%s_SoilGrids_percentage.tif' %SL)
    filename_om = os.path.join(Dir, 'SoilGrids', 'Soil_Organic_Carbon_Content' ,'SoilOrganicCarbonContent_%s_SoilGrids_g_kg.tif' %SL)
    #filename_bulkdensity = os.path.join(Dir, 'SoilGrids', 'Bulk_Density' ,'BulkDensity_%s_SoilGrids_kg-m-3.tif' %SL)
    
    # Define path for output
    if SL == "sl3":
        level = "Topsoil"
    elif SL == "sl6":
        level = "Subsoil"        

    filedir_out_densbulk = os.path.join( Dir, 'SoilGrids', 'Bulk_Density')
    if not os.path.exists(filedir_out_densbulk):
        os.makedirs(filedir_out_densbulk)
    filedir_out_thetasat = os.path.join(Dir, 'SoilGrids', 'Theta_Sat')
    if not os.path.exists(filedir_out_thetasat):
        os.makedirs(filedir_out_thetasat)    
    
    filename_out_densbulk = os.path.join(filedir_out_densbulk ,'Bulk_Density_%s_SoilGrids_g-cm-3.tif' %level)
    filename_out_thetasat = os.path.join(filedir_out_thetasat,'Theta_Sat_%s_SoilGrids_kg-kg.tif' %level)

    if not (os.path.exists(filename_out_densbulk) and os.path.exists(filename_out_thetasat)):
    #if not os.path.exists(filename_out_thetasat):
        
        # Open datasets
        dest_clay = gdal.Open(filename_clay)
        dest_om = gdal.Open(filename_om)
        #dest_bulk = gdal.Open(filename_bulkdensity)
    
        # Open Array info
        geo_out, proj, size_X, size_Y = RC.Open_array_info(filename_clay)    
        
        # Open Arrays
        Clay = dest_clay.GetRasterBand(1).ReadAsArray()
        OM = dest_om.GetRasterBand(1).ReadAsArray()
        Clay = np.float_(Clay)
        Clay[Clay>100]=np.nan
        OM = np.float_(OM)
        OM[OM<0]=np.nan  
        OM = OM/1000
        
        
        # Calculate bulk density
        bulk_dens = 1/(0.6117 + 0.3601 * Clay/100 + 0.002172 * np.power(OM * 100, 2)+ 0.01715 * np.log(OM * 100))
        #bulk_dens = dest_bulk.GetRasterBand(1).ReadAsArray()
        
        # Calculate theta saturated
        theta_sat = 0.85 * (1- (bulk_dens/2.65)) + 0.13 * Clay/100
        
        # Save data
        DC.Save_as_tiff(filename_out_densbulk, bulk_dens, geo_out, "WGS84")
        DC.Save_as_tiff(filename_out_thetasat, theta_sat, geo_out, "WGS84")
        
    return()    




# -*- coding: utf-8 -*-
import sys
from watertools.Collect.MSWEP.DataAccess import DownloadData


def main(Dir, Startdate, Enddate, latlim, lonlim, cores=False, Waitbar = 1):
    """
    This function downloads MSWEP precipitation daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """

    if Waitbar == 1:
        print('\nDownloading daily MSWEP precipitation data for the period %s till %s' %( Startdate, Enddate))

    # Download data
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores, TimeCase='daily')

if __name__ == '__main__':
    main(sys.argv)




# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''

# import watertools modules
from watertools.Collect import DEM
from watertools.Collect import CFSR
from watertools.Collect import GLDAS

def CollectData(Dir, Startdate, Enddate, latlim, lonlim, cores, LANDSAF):
    """
    This function Collect all the data needed for the ETref, by using wa.Collect functions.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- amount of cores used for collecting data
    LANDSAF -- if 1 than LANDSAF data will be used instead of CFSR data
    """
    # download DEM map (will only be done when it is not there yet)
    DEM.HydroSHED(Dir, latlim, lonlim, Waitbar = 0)

    if LANDSAF == 0:
        # download CFSR data (will only be done when it is not there yet)
        CFSR.daily(Dir=Dir, Vars = ['dlwsfc','dswsfc','ulwsfc'], Startdate=Startdate, Enddate=Enddate, latlim=latlim, lonlim=lonlim, Waitbar = 0)

    # Make latlim and lonlim a bit larger for GLDAS
    latlimGLDAS = [latlim[0] - 0.25, latlim[1] + 0.25]
    lonlimGLDAS = [lonlim[0] - 0.25, lonlim[1] + 0.25]

    # download GLDAS data (will only be done when it is not there yet)
    GLDAS.daily(Dir=Dir, Vars = ['tair_f_inst'], Startdate=Startdate, Enddate=Enddate, latlim=latlimGLDAS, lonlim=lonlimGLDAS, SumMean=0, Min=1, Max=1, Waitbar = 0)
    GLDAS.daily(Dir=Dir, Vars = ['psurf_f_inst','wind_f_inst','qair_f_inst'], Startdate=Startdate, Enddate=Enddate, latlim=latlimGLDAS, lonlim=lonlimGLDAS, SumMean=1, Min=0, Max=0, Waitbar = 0)

    return














# -*- coding: utf-8 -*-
"""
Created on Mon Jun 19 10:09:38 2017

@author: tih
"""

import os
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC
import numpy as np
import pandas as pd

def Calc_Humidity(Temp_format, P_format, Hum_format, output_format, Startdate, Enddate, freq ="D"):

    folder_dir_out = os.path.dirname(output_format)
    
    if not os.path.exists(folder_dir_out):
        os.makedirs(folder_dir_out)

    Dates = pd.date_range(Startdate, Enddate, freq = freq)
    
    for Date in Dates:
        
        try:
            print(Date)
            
            Day = Date.day
            Month = Date.month
            Year = Date.year
            Hour = Date.hour
            Minute = Date.minute
            
            Tempfile_one = Temp_format.format(yyyy = Year, mm = Month, dd = Day, HH = Hour, MM = Minute)
            Presfile_one = P_format.format(yyyy = Year, mm = Month, dd = Day, HH = Hour, MM = Minute)
            Humfile_one = Hum_format.format(yyyy = Year, mm = Month, dd = Day, HH = Hour, MM = Minute)
            out_folder_one = output_format.format(yyyy = Year, mm = Month, dd = Day, HH = Hour, MM = Minute)
        
            geo_out, proj, size_X, size_Y = RC.Open_array_info(Tempfile_one)
            Tdata = RC.Open_tiff_array(Tempfile_one)
            Tdata[Tdata<-900]=-9999
            Pdata = RC.Open_tiff_array(Presfile_one)
            Hdata = RC.Open_tiff_array(Humfile_one)
            Pdata[Pdata<0]=-9999
            Hdata[Hdata<0]=-9999
            
            # gapfilling
            Tdata = RC.gap_filling(Tdata,-9999)
            Pdata = RC.gap_filling(Pdata,-9999)
            Hdata = RC.gap_filling(Hdata,-9999)
            
            if "_K_" in Temp_format:
                Tdata = Tdata - 273.15

            Esdata = 0.6108*np.exp((17.27*Tdata)/(Tdata+237.3))
            HumData = np.minimum((1.6077717*Hdata*Pdata/Esdata),1)*100
            HumData = HumData.clip(0,100)
                            
            DC.Save_as_tiff(out_folder_one,HumData,geo_out,"WGS84") 
            print("FINISHED relative humidity for %s" %Date)
        except:
            print("NO relative humidity for %s" %Date)

    return()   


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/TWC

Description:
This module downloads the global gray water footprint data from
ftp.wateraccounting.unesco-ihe.org. Use the TWC.Gray_Water_Footprint function to
download the dataset.

Requirement:
The WA_FTP username and password must be filled in the WebAccounts.py.
Contact the Water Accounting Team to get access to our WA FTP server.

Developers:
This dataset is developed by the Twente Water Centre:
Mekonnen, Mesfin M., and Arjen Y. Hoekstra.
"Global gray water footprint and water pollution levels related to anthropogenic nitrogen loads to fresh water."
Environmental science & technology 49.21 (2015): 12860-12868.

Examples:
from watertools.Collect import TWC
TWC.Gray_Water_Footprint(Dir='C:/Temp/', latlim=[-10, 30], lonlim=[-20, -10])
"""

from .Gray_Water_Footprint import main as Gray_Water_Footprint

__all__ = ['Gray_Water_Footprint']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Created on Mon Aug 28 07:54:17 2017

@author: tih
"""
import sys
from watertools.Collect.SSEBop.DataAccess import DownloadData

def main(Dir, Startdate='', Enddate='', latlim=[-59.17, 80], lonlim=[-180, 180], version = "V4", Waitbar = 1):
    """
    This function downloads daily FEWS Potential Evapotranspiration data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -59.17 and 80)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    print('\nDownload daily FEWS potential evapotranspiration data for the period %s till %s' %(Startdate, Enddate))

    # Download data
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, version, Product = "ETpot")

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
    Authors: Tim Hessels
    Module: Products/GridSoils

    Description:
    This product will calculate soil properties by using the GridSoils as basis. 
	Different soil characteristics can be estimated.
	The formulas are taken from the SoMoi model and the SoilGrids are taken from:
	ftp.soilgrids.org
"""

from watertools.Products.SoilGrids import K_Sat
from watertools.Products.SoilGrids import Theta_FC
from watertools.Products.SoilGrids import Theta_Sat
from watertools.Products.SoilGrids import Theta_Sat2
from watertools.Products.SoilGrids import Theta_Res
from watertools.Products.SoilGrids import Water_Holding_Capacity
from watertools.Products.SoilGrids import n_van_genuchten

__all__ = ['K_Sat', 'Theta_FC', 'Theta_Sat', 'Theta_Sat2', 'Theta_Res', 'Water_Holding_Capacity', 'n_van_genuchten']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/SEBS

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the SEBS developers.

Description:
This script collects SEBS data from the UNESCO-IHE FTP server. The data has a
monthly temporal resolution and a spatial resolution of 0.01 degree. The
resulting tiff files are in the WGS84 projection.
The data is available between 2000-03-01 till 2015-12-31.

Example:
from watertools.Collect import SEBS
SEBS.monthly(Dir='C:/Temp/', Startdate='2003-02-24', Enddate='2003-03-09',
                     latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
import pandas as pd
from ftplib import FTP
import scipy.io as spio

# Water Accounting Modules
import watertools
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar):
    """
    This scripts downloads SEBS ET data from the UNESCO-IHE ftp server.
    The output files display the total ET in mm for a period of one month.
    The name of the file corresponds to the first day of the month.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    lonlim -- [ymin, ymax] (values must be between -90 and 90)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -90 or latlim[1] > 90:
        print('Latitude above 90N or below -90S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0], -90)
        latlim[1] = np.min(latlim[1], 90)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0],-180)
        lonlim[1] = np.min(lonlim[1],180)

	# Check Startdate and Enddate
    if not Startdate:
        Startdate = pd.Timestamp('2000-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('2017-06-30')

    # Creates dates library
    Dates = pd.date_range(Startdate, Enddate, freq = "MS")

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Define directory and create it if not exists
    output_folder = os.path.join(Dir, 'Evaporation', 'SEBS', 'Monthly')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    for Date in Dates:

        # Define year and month
        year = Date.year
        month = Date.month

        # Date as printed in filename
        Filename_out= os.path.join(output_folder,'ETa_SEBS_mm-month-1_monthly_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))

        # Define end filename
        Filename_in = os.path.join("ETm%d%02d.mat" %(year, month))

		 # Temporary filename for the downloaded global file
        local_filename = os.path.join(output_folder, Filename_in)

        # Download the data from FTP server if the file not exists
        if not os.path.exists(Filename_out):
            try:
                Download_SEBS_from_WA_FTP(local_filename, Filename_in)

                # Clip dataset
                Clip_Dataset(local_filename, Filename_out, latlim, lonlim)
                os.remove(local_filename)

            except:
                print("Was not able to download file with date %s" %Date)

        # Adjust waitbar
        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    return

def Download_SEBS_from_WA_FTP(local_filename, Filename_in):
    """
    This function retrieves SEBS data for a given date from the
    ftp.wateraccounting.unesco-ihe.org server.

    Restrictions:
    The data and this python file may not be distributed to others without
    permission of the WA+ team due data restriction of the SEBS developers.

    Keyword arguments:
	 local_filename -- name of the temporary file which contains global SEBS data
    Filename_in -- name of the end file with the monthly SEBS data
    """

    # Collect account and FTP information
    username, password = watertools.Functions.Random.Get_Username_PWD.GET('FTP_WA')
    ftpserver = "ftp.wateraccounting.unesco-ihe.org"

    # Download data from FTP
    ftp=FTP(ftpserver)
    ftp.login(username,password)
    directory="/WaterAccounting_Guest/SEBS/Global_land_ET_V1/"
    ftp.cwd(directory)
    lf = open(local_filename, "wb")
    ftp.retrbinary("RETR " + Filename_in, lf.write)
    lf.close()

    return

def Clip_Dataset(local_filename, Filename_out, latlim, lonlim):

    # Open Dataset
    SEBS_Array = spio.loadmat(local_filename)['ETm']

    # Define area
    XID = [int(np.floor((180 + lonlim[0])/0.05)), int(np.ceil((180 + lonlim[1])/0.05))]
    YID = [int(np.ceil((90 - latlim[1])/0.05)), int(np.floor((90 -  latlim[0])/0.05))]

    # Define Georeference
    geo = tuple([-180 + 0.05*XID[0],0.05,0,90 - 0.05*YID[0],0,-0.05])

    # Clip Array
    SEBS_Array_clipped = SEBS_Array[YID[0]:YID[1], XID[0]:XID[1]] * 0.1

    # Save tiff file
    DC.Save_as_tiff(Filename_out, SEBS_Array_clipped, geo, "WGS84")



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Products/ETens
"""
# General Modules
import sys
import pandas as pd
import numpy as np

# WA+ modules
from watertools.Products.ETens.DataAccess import DownloadData

def main(Dir, Startdate, Enddate, latlim, lonlim, Waitbar = 1):
    """
    This function downloads ETensemble data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """

	# Check the start and enddate
    if not Startdate:
        Startdate = pd.Timestamp('2003-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('2014-12-31')

    # Check and adjust if needed the latitude and longitude
    if latlim[0] < -90 or latlim[1] > 90:
        print('Latitude above 90N or below 90S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0],-90)
        latlim[1] = np.min(latlim[1],90)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0],-180)
        lonlim[1] = np.min(lonlim[1],180)

    print('/nCreate monthly ETensemble maps for period %s till %s' %(Startdate, Enddate))
    # Download and create the ET data
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar)

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/SRTM
"""

# General modules
import numpy as np
import os
import urllib
import shutil
from osgeo import gdal
import sys
import socket

# WA+ modules
import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC

def DownloadData(output_folder, latlim, lonlim):
    """
    This function downloads DEM data from SRTM

    Keyword arguments:
    output_folder -- directory of the result
	latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    """
   # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -60 or latlim[1] > 60:
        print('Latitude above 60N or below 60S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0], -60)
        latlim[1] = np.min(latlim[1], 60)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # converts the latlim and lonlim into names of the tiles which must be
    # downloaded
    name, rangeLon, rangeLat = Find_Document_Names(latlim, lonlim)

    # Memory for the map x and y shape (starts with zero)
    size_X_tot = 0
    size_Y_tot = 0

    nameResults = []
    
    # Create a temporary folder for processing
    output_folder_trash = os.path.join(output_folder, "Temp")
    if not os.path.exists(output_folder_trash):
        os.makedirs(output_folder_trash)

    # Download, extract, and converts all the files to tiff files
    for nameFile in name:

        try:
            # Download the data from
            # http://earlywarning.usgs.gov/hydrodata/
            output_file, file_name = Download_Data(nameFile, output_folder_trash)

            # extract zip data
            DC.Extract_Data(output_file, output_folder_trash)

            # The input is the file name and in which directory the data must be stored
            file_name_tiff = file_name.replace(".zip", ".tif")
            output_tiff = os.path.join(output_folder_trash, file_name_tiff)

            # convert data from adf to a tiff file
            dest_SRTM = gdal.Open(output_tiff)
            geo_out = dest_SRTM.GetGeoTransform()
            size_X = dest_SRTM.RasterXSize
            size_Y = dest_SRTM.RasterYSize            
   
            
            if (int(size_X) != int(6001) or int(size_Y) != int(6001)):
                data = np.ones((6001, 6001)) * -9999

                # Create the latitude bound
                Vfile = nameFile.split("_")[2][0:2]
                Bound2 = 60 - 5*(int(Vfile)-1)

                # Create the longitude bound
                Hfile = nameFile.split("_")[1]
                Bound1 = -180 + 5 * (int(Hfile)-1)

                Expected_X_min = Bound1
                Expected_Y_max = Bound2

                Xid_start = int(np.round((geo_out[0] - Expected_X_min)/geo_out[1]))
                Xid_end = int(np.round(((geo_out[0] + size_X * geo_out[1]) - Expected_X_min)/geo_out[1]))
                Yid_start = int(np.round((Expected_Y_max - geo_out[3])/(-geo_out[5])))
                Yid_end = int(np.round((Expected_Y_max - (geo_out[3] + (size_Y * geo_out[5])))/(-geo_out[5])))
                
                data_SRTM = dest_SRTM.GetRasterBand(1).ReadAsArray()

                data[Yid_start:Yid_end,Xid_start:Xid_end] = data_SRTM
                if np.max(data)==255:
                    data[data==255] = -9999
                data[data<-9999] = -9999

                geo_in = [Bound1 - 0.5 * 0.00083333333333333, 0.00083333333333333, 0.0, Bound2 + 0.5 * 0.00083333333333333,
                          0.0, -0.0008333333333333333333]

                # save chunk as tiff file
                destDEM = DC.Save_as_MEM(data=data, geo=geo_in, projection="WGS84")
                
                dest_SRTM = None 

        except:

            # If tile not exist create a replacing zero tile (sea tiles)
            file_name_tiff = file_name.replace(".zip", ".tif")
            output_tiff = os.path.join(output_folder_trash, file_name_tiff)
            file_name = nameFile
            data = np.ones((6001, 6001)) * -9999
            data = data.astype(np.float32)

            # Create the latitude bound
            Vfile = nameFile.split("_")[2][0:2]
            Bound2 = 60 - 5*(int(Vfile)-1)

            # Create the longitude bound
            Hfile = nameFile.split("_")[1]
            Bound1 = -180 + 5 * (int(Hfile)-1)

            # Geospatial data for the tile
            geo_in = [Bound1 - 0.5 * 0.00083333333333333, 0.00083333333333333, 0.0, Bound2 + 0.5 * 0.00083333333333333,
                          0.0, -0.0008333333333333333333]

            # save chunk as tiff file
            destDEM = DC.Save_as_MEM(data=data, geo=geo_in, projection="WGS84")


        # clip data
        Data, Geo_data, Proj_out = RC.clip_data(destDEM, latlim, lonlim)
        size_Y_out = int(np.shape(Data)[0])
        size_X_out = int(np.shape(Data)[1])

        # Total size of the product so far
        size_Y_tot = int(size_Y_tot + size_Y_out)
        size_X_tot = int(size_X_tot + size_X_out)

        if nameFile is name[0]:
            Geo_x_end = Geo_data[0]
            Geo_y_end = Geo_data[3]
        else:
            Geo_x_end = np.min([Geo_x_end,Geo_data[0]])
            Geo_y_end = np.max([Geo_y_end,Geo_data[3]])

        # create name for chunk
        FileNameEnd = "%s_temporary.tif" % (file_name)
        nameForEnd = os.path.join(output_folder_trash, FileNameEnd)
        nameResults.append(str(nameForEnd))

        # save chunk as tiff file
        DC.Save_as_tiff(name=nameForEnd, data=Data, geo=Geo_data,
                      projection="WGS84")


    size_X_end = int(size_X_tot/len(rangeLat)) # + 1 !!!
    size_Y_end = int(size_Y_tot/len(rangeLon)) # + 1 !!!

    # Define the georeference of the end matrix
    geo_out = [Geo_x_end, Geo_data[1], 0, Geo_y_end, 0, Geo_data[5]]

    latlim_out = [geo_out[3] + geo_out[5] * size_Y_end, geo_out[3]]
    lonlim_out = [geo_out[0], geo_out[0] + geo_out[1] * size_X_end]


    # merge chunk together resulting in 1 tiff map
    datasetTot = Merge_DEM(latlim_out, lonlim_out, nameResults, size_Y_end,
                                size_X_end)

    datasetTot[datasetTot<-9999] = -9999


    # name of the end result
    output_DEM_name = "DEM_SRTM_m_3s.tif" 
    
    Save_name = os.path.join(output_folder, output_DEM_name)

    # Make geotiff file
    DC.Save_as_tiff(name=Save_name, data=datasetTot, geo=geo_out, projection="WGS84")
    os.chdir(output_folder)

    # Delete the temporary folder
    #shutil.rmtree(output_folder_trash)


def Merge_DEM(latlim, lonlim, nameResults, size_Y_tot, size_X_tot):
    """
    This function will merge the tiles

    Keyword arguments:
    latlim -- [ymin, ymax], (values must be between -50 and 50)
    lonlim -- [xmin, xmax], (values must be between -180 and 180)
    nameResults -- ['string'], The directories of the tiles which must be
                   merged
    size_Y_tot -- integer, the width of the merged array
    size_X_tot -- integer, the length of the merged array
    """
    # Define total size of end dataset and create zero array
    datasetTot = np.ones([size_Y_tot, size_X_tot])*-9999.

    # Put all the files in the datasetTot (1 by 1)
    for nameTot in nameResults:
        f = gdal.Open(nameTot)
        dataset = np.array(f.GetRasterBand(1).ReadAsArray())
        dataset = np.flipud(dataset)
        geo_out = f.GetGeoTransform()
        BoundChunk1 = int(round((geo_out[0]-lonlim[0])/geo_out[1]))
        BoundChunk2 = BoundChunk1 + int(dataset.shape[1])
        BoundChunk4 = size_Y_tot + int(round((geo_out[3] -
                                       latlim[1]) / geo_out[1]))
        BoundChunk3 = BoundChunk4 - int(dataset.shape[0])
        datasetTot[BoundChunk3:BoundChunk4, BoundChunk1:BoundChunk2] = dataset
        f = None
    datasetTot = np.flipud(datasetTot)
    return(datasetTot)

def Find_Document_Names(latlim, lonlim):
    """
    This function will translate the latitude and longitude limits into
    the filenames that must be downloaded from the hydroshed webpage

    Keyword Arguments:
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # find tiles that must be downloaded
    startLat = np.floor((60 - latlim[1])/ 5) + 1
    startLon = np.floor((180 + lonlim[0])/ 5) + 1
    endLat = np.ceil((60 - latlim[0])/ 5.0) + 1
    endLon = np.ceil((180 + lonlim[1])/ 5.0) + 1
    rangeLon = np.arange(startLon, endLon, 1)
    rangeLat = np.arange(startLat, endLat, 1)

    name = []

    # make the names of the files that must be downloaded
    for lonname in rangeLon:
        for latname in rangeLat:
            name.append(str("srtm_%02d_%02d.zip" %(lonname, latname)))
            
    return(name, rangeLon, rangeLat)


def Download_Data(nameFile, output_folder_trash):
    """
    This function downloads the DEM data from the HydroShed website

    Keyword Arguments:
    nameFile -- name, name of the file that must be downloaded
    output_folder_trash -- Dir, directory where the downloaded data must be
                           stored
    """

    # download data from the internet
    url= "http://srtm.csi.cgiar.org/wp-content/uploads/files/srtm_5x5/TIFF/%s" %(nameFile) 
    
    try:    
        socket.setdefaulttimeout(300)
        file_name = url.split('/')[-1]
        output_file = os.path.join(output_folder_trash, file_name)
        if sys.version_info[0] == 3:
            urllib.request.urlretrieve(url, output_file)
        if sys.version_info[0] == 2:
            urllib.urlretrieve(url, output_file)
    except:
        pass

    return(output_file, file_name)


def Find_Document_names_15s_30s(latlim, lonlim, parameter, resolution):

    continents = ['na','ca','sa','eu','af','as','au']
    continents_download = []

    for continent in continents:
        extent = DEM_15s_extents.Continent[continent]
        if (extent[0] < lonlim[0] and extent[1] > lonlim[0] and extent[2] < latlim[0] and extent[3] > latlim[0]) and (extent[0] < lonlim[1] and extent[1] > lonlim[1] and extent[2] < latlim[1] and extent[3] > latlim[1]) == True:
            if resolution == "15s":
                name = '%s_%s_%s_grid.zip' %(continent, parameter, resolution)
            if resolution =="30s":
                name = '%s_%s_%s_bil.zip' %(continent, parameter, resolution)                
            continents_download = np.append(continents_download,name)

    return(continents_download)

class DEM_15s_extents:
        Continent = { 'na' : [-138, -52, 24, 60],
					'ca' : [-119, -60, 5, 39],
                      'sa' : [-93, -32, -56, 15],
                      'eu' : [-14, 70, 12, 62],
                      'af' : [-19, 55, -35, 38],
                      'as' : [57, 180, -12, 61],
                      'au' : [112, 180, -56, -10]}


# -*- coding: utf-8 -*-
import sys
import pandas as pd
import calendar
import os

from watertools.Collect.RFE.DataAccess import DownloadData
import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC

def main(Dir, Startdate='', Enddate='',
         latlim=[-50, 50], lonlim=[-180, 180], cores=False, Waitbar = 1):

    """
    This function downloads RFE V2.0 (monthly) data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine.
             It can be 'False' to avoid using parallel computing
             routines.
    Waitbar -- 1 (Default) will print a waitbar
    """
     # Download data
    print('\nDownload monthly RFE precipitation data for period %s till %s' %(Startdate, Enddate))

    # Check variables
    if not Startdate:
        Startdate = pd.Timestamp('2001-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('Now')
    Dates = pd.date_range(Startdate,  Enddate, freq='MS')

	 # Make directory
    output_folder = os.path.join(Dir, 'Precipitation', 'RFE', 'Monthly/')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    for Date in Dates:
        month = Date.month
        year = Date.year
        end_day = calendar.monthrange(year, month)[1]
        Startdate_one_month = '%s-%02s-01' %(year, month)
        Enddate_one_month = '%s-%02s-%02s' %(year, month, end_day)

        DownloadData(Dir, Startdate_one_month, Enddate_one_month, latlim, lonlim, 0, cores)

        Dates_daily = pd.date_range(Startdate_one_month,  Enddate_one_month, freq='D')

	     # Make directory
        input_folder_daily = os.path.join(Dir, 'Precipitation', 'RFE', 'Daily/')
        i = 0

        for Date_daily in Dates_daily:
            file_name = 'P_RFE.v2.0_mm-day-1_daily_%s.%02s.%02s.tif' %(Date_daily.strftime('%Y'), Date_daily.strftime('%m'), Date_daily.strftime('%d'))
            file_name_daily_path = os.path.join(input_folder_daily, file_name)
            if os.path.exists(file_name_daily_path):
                if Date_daily == Dates_daily[i]:
                    Raster_monthly = RC.Open_tiff_array(file_name_daily_path)
                else:
                    Raster_monthly += RC.Open_tiff_array(file_name_daily_path)
            else:
                if Date_daily == Dates_daily[i]:
                    i += 1

        geo_out, proj, size_X, size_Y = RC.Open_array_info(file_name_daily_path)
        file_name = 'P_RFE.v2.0_mm-month-1_monthly_%s.%02s.01.tif' %(Date.strftime('%Y'), Date.strftime('%m'))
        file_name_output = os.path.join(output_folder, file_name)
        DC.Save_as_tiff(file_name_output, Raster_monthly, geo_out, projection="WGS84")

        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

if __name__ == '__main__':
    main(sys.argv)
